/*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

//#define DEBUG
//#define SYS_WIN32

#include <malloc.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef SYS_WIN32
    #include <fcntl.h>
    #include <io.h>
#endif

//-----------------------------------------------------------------------------------------------------
//===< MAKRO >=========================================================================================
//-----------------------------------------------------------------------------------------------------

//data type
#define BYTE   unsigned char
#define SBYTE  signed char
#define WORD   unsigned short int
#define SWORD  signed short int
#define DWORD  unsigned int
#define SDWORD signed int

//jfif marker
#define SOF0  0xC0 //start of frame: baseline
#define SOF1  0xC1 //start of frame: extended sequential
#define SOF2  0xC2 //start of frame: progressive
#define SOF3  0xC3 //start of frame: lossless
#define DHT   0xC4 //define huffman table
#define SOF5  0xC5 //start of frame: differential sequential
#define SOF6  0xC6 //start of frame: differential progressive
#define SOF7  0xC7 //start of frame: differential lossless
#define SOF9  0xC8 //start of frame: extended sequential, arithmetic coding
#define SOF10 0xCA //start of frame: progressive, arithmetic coding
#define SOF11 0xCB //start of frame: lossless, arithmetic coding
#define SOF13 0xCD //start of frame: differential sequential, arithmetic coding
#define SOF14 0xCE //start of frame: differential progressive, arithmetic coding
#define SOF15 0xCF //start of frame: differential lossless, arithmetic coding
#define SOI   0xD8 //start of image
#define EOI   0xD9 //end of image
#define SOS   0xDA //start of scan
#define DQT   0xDB //define quantization table
#define DRI   0xDD //define restart interval
#define APP0  0xE0 //application
#define COM   0xFE //comment

//quality
#define QUALITY_LUMINANCE     25
#define QUALITY_CHROMINANCE   20
#define THRESHOLD_LUMINANCE    5
#define THRESHOLD_CHROMINANCE 15

//file access
#define GET_BYTE()  data_byte = data[data_position++]
#define GET_WORD()  data_word = ((WORD)data[data_position] << 8) | data[data_position + 1]; data_position += 2
#define END_OF_DATA (data_position > data_length)
#define PUT_BYTE(i) fputc((i), file_handle)
#define PUT_WORD(i) PUT_BYTE((i) >> 8); \
                    PUT_BYTE((i) & 0x00FF)

//conversion
#define BYTE_HIGH(i)        ((i) >> 4)
#define BYTE_LOW(i)         ((i) & 0x0F)
#define BYTE_HIGH_LOW(i, j) (((i) << 4) | (j))
#define WORD_HIGH_LOW(i, j) (((i) << 8) | (j))
#define CONVERT_SWORD(i, j) (SWORD)(((i) < bitmask2[j]) ? (i) - bitmask1[j] : (i))
#define CONVERT_WORD(i, j)  (WORD) (((i) < bitmask2[j]) ? (i) + bitmask1[j] : (i))

//debug
#ifdef DEBUG
    #define ERROR(s) {fprintf(stderr, "\ncritical error: %s\n", s); exit(0);}
#else
    #define ERROR(s) exit(0);
#endif

//-----------------------------------------------------------------------------------------------------
//===< GLOBAL >========================================================================================
//-----------------------------------------------------------------------------------------------------

//comment
const char comment[50] = "generated by nuos CMS, http://www.nuos-cms.org";

//standard luminance quantization table
static BYTE std_lqtable[64] = {16,  11,  10,  16,  24,  40,  51,  61,
                               12,  12,  14,  19,  26,  58,  60,  55,
                               14,  13,  16,  24,  40,  57,  69,  56,
                               14,  17,  22,  29,  51,  87,  80,  62,
                               18,  22,  37,  56,  68, 109, 103,  77,
                               24,  35,  55,  64,  81, 104, 113,  92,
                               49,  64,  78,  87, 103, 121, 120, 101,
                               72,  92,  95,  98, 112, 100, 103,  99};

//standard chrominance quantization table
static BYTE std_cqtable[64] = {17,  18,  24,  47,  99,  99,  99,  99,
                               18,  21,  26,  66,  99,  99,  99,  99,
                               24,  26,  56,  99,  99,  99,  99,  99,
                               47,  66,  99,  99,  99,  99,  99,  99,
                               99,  99,  99,  99,  99,  99,  99,  99,
                               99,  99,  99,  99,  99,  99,  99,  99,
                               99,  99,  99,  99,  99,  99,  99,  99,
                               99,  99,  99,  99,  99,  99,  99,  99};

//standard huffman table
typedef struct
{
          BYTE count;
    WORD value;
} STD_HTABLE;

//standard dc luminance huffman table
static const BYTE std_dc_lhcount[16] = {0,  1,  5,  1,  1,  1,  1,  1,
                                        1,  0,  0,  0,  0,  0,  0,  0};
static const BYTE std_dc_lhvalue[12] = {0,  1,  2,  3,  4,  5,  6,  7,
                                        8,  9, 10, 11};
static STD_HTABLE std_dc_lhtable[12];

//standard dc chrominance huffman table
static const BYTE std_dc_chcount[16] = {0,  3,  1,  1,  1,  1,  1,  1,
                                        1,  1,  1,  0,  0,  0,  0,  0};
static const BYTE std_dc_chvalue[12] = {0,  1,  2,  3,  4,  5,  6,  7,
                                        8,  9, 10, 11};
static STD_HTABLE std_dc_chtable[12];

//standard ac luminance huffman table
static const BYTE std_ac_lhcount[16]  = {   0,    2,    1,    3,    3,    2,    4,    3,
                                            5,    5,    4,    4,    0,    0,    1,  125};
static const BYTE std_ac_lhvalue[162] = {0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12,
                                         0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07,
                                         0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xA1, 0x08,
                                         0x23, 0x42, 0xB1, 0xC1, 0x15, 0x52, 0xD1, 0xF0,
                                         0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0A, 0x16,
                                         0x17, 0x18, 0x19, 0x1A, 0x25, 0x26, 0x27, 0x28,
                                         0x29, 0x2A, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
                                         0x3A, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
                                         0x4A, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
                                         0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
                                         0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
                                         0x7A, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
                                         0x8A, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
                                         0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7,
                                         0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6,
                                         0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3, 0xC4, 0xC5,
                                         0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4,
                                         0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xE1, 0xE2,
                                         0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA,
                                         0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
                                         0xF9, 0xFA};
static STD_HTABLE std_ac_lhtable[251];

//standard ac chrominance huffman table
static const BYTE std_ac_chcount[16]  = {   0,    2,    1,    2,    4,    4,    3,    4,
                                            7,    5,    4,    4,    0,    1,    2,  119};
static const BYTE std_ac_chvalue[162] = {0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21,
                                         0x31, 0x06, 0x12, 0x41, 0x51, 0x07, 0x61, 0x71,
                                         0x13, 0x22, 0x32, 0x81, 0x08, 0x14, 0x42, 0x91,
                                         0xA1, 0xB1, 0xC1, 0x09, 0x23, 0x33, 0x52, 0xF0,
                                         0x15, 0x62, 0x72, 0xD1, 0x0A, 0x16, 0x24, 0x34,
                                         0xE1, 0x25, 0xF1, 0x17, 0x18, 0x19, 0x1A, 0x26,
                                         0x27, 0x28, 0x29, 0x2A, 0x35, 0x36, 0x37, 0x38,
                                         0x39, 0x3A, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48,
                                         0x49, 0x4A, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58,
                                         0x59, 0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
                                         0x69, 0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78,
                                         0x79, 0x7A, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87,
                                         0x88, 0x89, 0x8A, 0x92, 0x93, 0x94, 0x95, 0x96,
                                         0x97, 0x98, 0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5,
                                         0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4,
                                         0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3,
                                         0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2,
                                         0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA,
                                         0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9,
                                         0xEA, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
                                         0xF9, 0xFA};
static STD_HTABLE std_ac_chtable[251];

//huffman table
typedef struct
{
    BYTE count[16];
    BYTE min_count;
        WORD minor[16];
        WORD major[16];
    BYTE value[65536];
} HTABLE;
static HTABLE *htable_dc[] = {NULL, NULL, NULL, NULL};
static HTABLE *htable_ac[] = {NULL, NULL, NULL, NULL};

//quantization table
static BYTE qtable[3][64];

//serialization table
static const BYTE stable[] = { 0,  1,  5,  6, 14, 15, 27, 28,
                               2,  4,  7, 13, 16, 26, 29, 42,
                               3,  8, 12, 17, 25, 30, 41, 43,
                               9, 11, 18, 24, 31, 40, 44 ,53,
                              10, 19, 23, 32, 39, 45, 52, 54,
                              20, 22, 33, 38, 46, 51, 55, 60,
                              21, 34, 37, 47, 50, 56, 59, 61,
                              35, 36, 48, 49, 57, 58, 62, 63};

//bitmask table
static const DWORD bitmask1[] = {0x00000000,
                                 0x00000001, 0x00000003, 0x00000007, 0x0000000F,
                                 0x0000001F, 0x0000003F, 0x0000007F, 0x000000FF,
                                 0x000001FF, 0x000003FF, 0x000007FF, 0x00000FFF,
                                 0x00001FFF, 0x00003FFF, 0x00007FFF, 0x0000FFFF,
                                 0x0001FFFF, 0x0003FFFF, 0x0007FFFF, 0x000FFFFF,
                                 0x001FFFFF, 0x003FFFFF, 0x007FFFFF, 0x00FFFFFF,
                                 0x01FFFFFF, 0x03FFFFFF, 0x07FFFFFF, 0x0FFFFFFF,
                                 0x1FFFFFFF, 0x3FFFFFFF, 0x7FFFFFFF, 0xFFFFFFFF};
static const SWORD bitmask2[] = {0x0000, 0x0001, 0x0002, 0x0004,
                                 0x0008, 0x0010, 0x0020, 0x0040,
                                 0x0080, 0x0100, 0x0200, 0x0400,
                                 0x0800, 0x1000, 0x2000, 0x4000};

//bitstring
typedef struct
{
    BYTE count;
    DWORD value;
} BITSTRING;
static BITSTRING bitstring;

//buffer data
typedef struct
{
    WORD width;
    WORD height;
    SWORD *cmp[3];
    DWORD size;
} BUFFER;
static BUFFER buffer;
static DWORD buffer_position[64];

//target buffer data
typedef struct
{
    WORD width;
    WORD height;
    BYTE *cmp[3];
    DWORD size;
} TARGET;
static TARGET target;

//component data
static BYTE cmp_count;
typedef struct
{
    BYTE index;
    BYTE width;
    BYTE height;
    BYTE factor_x;
    BYTE factor_y;
    BYTE scale_x;
    BYTE scale_y;
    SWORD dc;
    BYTE *qtable;
    void *htable_dc;
    void *htable_ac;
} CMP;
static CMP cmp[3];
static BYTE cmp_max_factor_x = 0;
static BYTE cmp_max_factor_y = 0;

//scan data
typedef struct
{
    BYTE cmp_count;
    BYTE ss_start; //spectral selection start
    BYTE ss_end;   //spectral selection end
    BYTE sa_high;  //successive approximation high bit
    BYTE sa_low;   //successive approximation low bit
    WORD eobrun;   //end of band run count
    CMP *cmp[3];
} SCAN;
static SCAN scan;

//image data
static WORD img_width;
static WORD img_height;

//mcu data
static BYTE mcu_width;
static BYTE mcu_height;

//data access
static FILE *file_handle;
static BYTE *data;
static DWORD data_position;
static DWORD data_length;
static WORD data_word;
static BYTE data_byte;
static SWORD data_unit[64];

//flag
static BYTE flag_marker  = 0;
static BYTE flag_mode    = 0;
static BYTE flag_restart = 0;

//forward declaration
int main(int argc, char *argv[]); //(1:{input_file}) (2:{width}|*) (3:{height}|*) (4:{output_file})
void jpeg_free_resource(void);

//-----------------------------------------------------------------------------------------------------
//===< DECODER >=======================================================================================
//-----------------------------------------------------------------------------------------------------

int jpeg_decode(const char *file);

int jpeg_decoder_read_file(const char *file);
int jpeg_decoder_seek_marker(void);
void jpeg_decoder_read_htable(HTABLE *htable);
void jpeg_decoder_read_qtable(BYTE *qtable);
void jpeg_decoder_process_scan(void);
WORD jpeg_decoder_peek_bit(BYTE count);
WORD jpeg_decoder_get_bit(BYTE count);
void jpeg_decoder_clear_bit(BYTE count);
BYTE jpeg_decoder_get_hvalue(HTABLE *htable);
void jpeg_decoder_get_data_unit(BYTE index);
void jpeg_decoder_dequantize(BYTE index);
void jpeg_decoder_idct(void);

//-----------------------------------------------------------------------------------------------------
//===< PROGRESSIVE >===================================================================================
//-----------------------------------------------------------------------------------------------------

void jpeg_pdecoder_process_scan(void);
void jpeg_pdecoder_process_data(void);
void jpeg_pdecoder_dc_base(BYTE index, SWORD *buffer);
void jpeg_pdecoder_dc_approximation(BYTE index, SWORD *buffer);
void jpeg_pdecoder_ac_base(BYTE index, SWORD *buffer);
void jpeg_pdecoder_ac_approximation(BYTE index, SWORD *buffer);
void jpeg_pdecoder_dequantize(BYTE index, SWORD *buffer);

//-----------------------------------------------------------------------------------------------------
//===< RESAMPLER >=====================================================================================
//-----------------------------------------------------------------------------------------------------

int jpeg_resample(void);
BYTE jpeg_resampler_cubic(BYTE value1, BYTE value2, BYTE value3, BYTE value4, double fraction);

//-----------------------------------------------------------------------------------------------------
//===< ENCODER >=======================================================================================
//-----------------------------------------------------------------------------------------------------

int jpeg_encode(const char *file);

void jpeg_encoder_create_bitcode(void);
void jpeg_encoder_create_htable(STD_HTABLE *htable, const BYTE *hcount, const BYTE *hvalue);
void jpeg_encoder_put_bit(BYTE count, WORD value);
void jpeg_encoder_put_data_unit(BYTE index);
void jpeg_encoder_quantize(BYTE index);
void jpeg_encoder_fdct(void);

//-----------------------------------------------------------------------------------------------------
//===< MAIN >==========================================================================================
//-----------------------------------------------------------------------------------------------------

int main(int argc, char *argv[])
{
    //target image size
    if (argc < 3) ERROR("syntax error");
    target.width  = (*argv[2] == 0x2A)             ? 0 : atoi(argv[2]);
    target.height = (argc < 4 || *argv[3] == 0x2A) ? 0 : atoi(argv[3]);

    atexit(jpeg_free_resource);

    if (jpeg_decode(argv[1]))
    {
        if (jpeg_resample())
        {
            if (jpeg_encode((argc >= 5) ? argv[4] : NULL)) return 1;
        };
    };

    return 0;
};
//-----------------------------------------------------------------------------------------------------
void jpeg_free_resource(void)
{
    int i;

    if (data) free(data);
    for (i = 0; i < 3; i++)
    {
        if (buffer.cmp[i]) free(buffer.cmp[i]);
        if (target.cmp[i]) free(target.cmp[i]);
        if (htable_dc[i])  free(htable_dc[i]);
        if (htable_ac[i])  free(htable_ac[i]);
    };
};

//-----------------------------------------------------------------------------------------------------
//===< DECODER >=======================================================================================
//-----------------------------------------------------------------------------------------------------

int jpeg_decode(const char *file)
{
    DWORD _data_position;
    DWORD __data_position;
    DWORD i;
    WORD length;
    BYTE index;
    BYTE found_app0 = 0,
         found_dht  = 0,
         found_dqt  = 0,
         found_sofn = 0,
         found_sos  = 0;

    //read file
    if (! jpeg_decoder_read_file(file)) return 0;

    //find SOI marker
    if (
         (! jpeg_decoder_seek_marker())
         ||
         (data_byte != SOI)
       )
        ERROR("invalid file format");

    while (! END_OF_DATA)
    {
        //find next marker
        if (! jpeg_decoder_seek_marker()) ERROR("invalid file format");
        if (data_byte == EOI)
        {
            if (found_sos)
            {
                if (flag_mode == SOF2) jpeg_pdecoder_process_data();
                return 1;
            };

            return 0;
        };

        //get header length
        GET_WORD();
        length         = data_word - 2;
        _data_position = data_position;

        switch (data_byte)
        {
        case APP0:
//.....................................................................................................
            found_app0 = 1;

            //validate file type
            if (
                 (data[data_position + 0] != 0x4A) //J
                 ||
                 (data[data_position + 1] != 0x46) //F
                 ||
                 (data[data_position + 2] != 0x49) //I
                 ||
                 (data[data_position + 3] != 0x46) //F
                 ||
                 (data[data_position + 4] != 0x00) //zero termination
                 ||
                 (data[data_position + 5] != 0x01) //version 1.x
               )
                ERROR("invalid file format");

            break;

        case DHT:
//.....................................................................................................
            found_dht = 1;

            for (i = 0; i < length; i += data_position - __data_position)
            {
                __data_position = data_position;
                GET_BYTE();
                index           = BYTE_LOW(data_byte);
                if (index > 3) ERROR("invalid huffman table index");
                if (data_byte & 0x10)
                {
                    if (! htable_ac[index]) htable_ac[index] = (HTABLE *)malloc(sizeof(HTABLE));
                    if (! htable_ac[index]) ERROR("out of memory");
                    jpeg_decoder_read_htable(htable_ac[index]);
                }
                else
                {
                    if (! htable_dc[index]) htable_dc[index] = (HTABLE *)malloc(sizeof(HTABLE));
                    if (! htable_dc[index]) ERROR("out of memory");
                    jpeg_decoder_read_htable(htable_dc[index]);
                };
            };

            break;

        case DQT:
//.....................................................................................................
            found_dqt = 1;

            for (i = 0; i < length; i += data_position - __data_position)
            {
                __data_position = data_position;
                GET_BYTE();
                if (data_byte & 0xF0) ERROR("precision not supported");
                if (data_byte > 2)    ERROR("invalid quantization table index");
                jpeg_decoder_read_qtable(qtable[data_byte]);
            };

            break;

        case SOF0:  //baseline
        case SOF1:  //extended sequential
        case SOF2:  //progressive
        case SOF3:  //lossless
        case SOF5:  //differential sequential
        case SOF6:  //differential progressive
        case SOF7:  //differential lossless
        case SOF9:  //extended sequential, arithmetic coding
        case SOF10: //progressive, arithmetic coding
        case SOF11: //lossless, arithmetic coding
        case SOF13: //differential sequential, arithmetic coding
        case SOF14: //differential progressive, arithmetic coding
        case SOF15: //differential lossless, arithmetic coding
//.....................................................................................................
            found_sofn     = 1;
            flag_mode      = data_byte;

            //mode
            if (flag_mode != SOF0 && flag_mode != SOF2) ERROR("mode not supported");

            //precision
            GET_BYTE();
            if (data_byte != 8)                         ERROR("precision not supported");

            //image height
            GET_WORD();
            img_height     = data_word;

            //image width
            GET_WORD();
            img_width      = data_word;

            //component count
            GET_BYTE();
            if (data_byte != 1 && data_byte != 3)       ERROR("component model not supported");
            cmp_count      = data_byte;

            //component data
            for (i = 0; i < cmp_count; i++)
            {
                __data_position     = data_position;
                GET_BYTE();
                if (! data_byte || data_byte > 3)           ERROR("invalid component index");
                index               = data_byte - 1;        //component id
                cmp[index].index    = index;
                GET_BYTE();
                cmp[index].factor_x = BYTE_HIGH(data_byte); //horizontal sampling factor
                cmp[index].factor_y = BYTE_LOW(data_byte);  //vertical sampling factor
                GET_BYTE();
                cmp[index].qtable   = qtable[data_byte];    //quantization table id

                //maximum scaling factor
                if (cmp[index].factor_x > cmp_max_factor_x) cmp_max_factor_x = cmp[index].factor_x;
                if (cmp[index].factor_y > cmp_max_factor_y) cmp_max_factor_y = cmp[index].factor_y;
            };

            //component scaling factor
            for (i = 0; i < cmp_count; i++)
            {
                cmp[i].scale_x      = cmp_max_factor_x / cmp[i].factor_x;
                cmp[i].scale_y      = cmp_max_factor_y / cmp[i].factor_y;
                cmp[i].width        = cmp[i].factor_x * 8;
                cmp[i].height       = cmp[i].factor_y * 8;
            };

            //mcu size
            mcu_width      = cmp_max_factor_x * 8;
            mcu_height     = cmp_max_factor_y * 8;

            //buffer
            buffer.width   = img_width  % mcu_width;
            if (buffer.width)                           buffer.width  = mcu_width  - buffer.width;
            buffer.width  += img_width;

            buffer.height  = img_height % mcu_height;
            if (buffer.height)                          buffer.height = mcu_height - buffer.height;
            buffer.height += img_height;

            buffer.size    = buffer.width * buffer.height * sizeof(SWORD);

            for (i = 0; i < cmp_count; i++)
            {
                buffer.cmp[i]       = (SWORD *)malloc(buffer.size);
                if (buffer.cmp[i] == NULL)                  ERROR("out of memory");
                memset(buffer.cmp[i], 0, buffer.size);
            };

            for (i = 0; i < 64; i++) buffer_position[i] = i / 8 * buffer.width + i % 8;

            break;

        case SOS:
//.....................................................................................................
            /* APP0 header check disabled due to
               some digital cameras known to omit it */
            //if (! found_app0) ERROR("missing APP0 header");
            if (! found_dht)  ERROR("missing DHT header");
            if (! found_dqt)  ERROR("missing DQT header");
            if (! found_sofn) ERROR("missing SOFn header");

            found_sos = 1;

            //component count
            GET_BYTE();
            scan.cmp_count  = data_byte;
            if (scan.cmp_count > cmp_count) ERROR("invalid number of components");

            //component data
            for (i = 0; i < scan.cmp_count; i++)
            {
                GET_BYTE();
                if (! data_byte || data_byte > 3) ERROR("invalid component index");
                index                = data_byte - 1;                   //component id
                cmp[index].dc        = 0;
                GET_BYTE();
                cmp[index].htable_dc = htable_dc[BYTE_HIGH(data_byte)]; //dc huffman table id
                cmp[index].htable_ac = htable_ac[BYTE_LOW(data_byte)];  //ac huffman table id

                scan.cmp[i]          = &cmp[index];
            };

            //spectral selection start
            GET_BYTE();
            scan.ss_start   = data_byte;

            //spectral selection end
            GET_BYTE();
            scan.ss_end     = data_byte;

            //successive approximation high/low bit
            GET_BYTE();
            scan.sa_high    = BYTE_HIGH(data_byte);
            scan.sa_low     = BYTE_LOW(data_byte);

            data_position   = _data_position + length;

            //reset
            flag_marker     = 0;
            bitstring.count = 0;
            bitstring.value = 0;

            switch (flag_mode)
            {
            case SOF0:
                jpeg_decoder_process_scan();
                return 1;

            case SOF2:
                jpeg_pdecoder_process_scan();
                continue;
            };

            return 0;
        };

        data_position = _data_position + length;
    };

    return 0;
};
//-----------------------------------------------------------------------------------------------------
int jpeg_decoder_read_file(const char *file)
{
    //open file
    file_handle = fopen(file, "rb");
    if (file_handle == NULL) ERROR("file not found");

    //determine filesize
    fseek(file_handle, 0, SEEK_END);
    data_length = ftell(file_handle);

    //allocate buffer memory
    data        = (BYTE *)malloc(data_length);
    if (data == NULL)
    {
        fclose(file_handle);
        ERROR("out of memory");
    };

    //store file content in memory
    fseek(file_handle, 0, SEEK_SET);
    fread(data, 1, data_length, file_handle);
    fclose(file_handle);

    return 1;
};
//-----------------------------------------------------------------------------------------------------
int jpeg_decoder_seek_marker(void)
{
    do
    {
        do {if (END_OF_DATA) return 0; GET_BYTE();} while (data_byte != 0xFF);
        do {if (END_OF_DATA) return 0; GET_BYTE();} while (data_byte == 0xFF);
    }
    while (data_byte == 0x00);

    return ! END_OF_DATA;
};
//-----------------------------------------------------------------------------------------------------
void jpeg_decoder_read_htable(HTABLE *htable)
{
    WORD code = 0;
    BYTE count;
    BYTE i;

    //read code length
    for (count = 0; count < 16; count++)
    {
        GET_BYTE();
        htable->count[count] = data_byte;
    };

    //code bit range
    htable->min_count = 0;

    //read code value
    for (count = 0; count < 16; count++)
    {
        for (i = 0; i < htable->count[count]; i++)
        {
            GET_BYTE();
            htable->value[WORD_HIGH_LOW(count, i)] = data_byte;
        };

        //compute category range
        if (htable->count[count])
        {
            htable->minor[count]  = code;
            code                 += htable->count[count];
            htable->major[count]  = code - 1;

            //htable->max_count     = count + 1;
            if (! htable->min_count) htable->min_count = count + 1;
        }
        else
        {
            htable->minor[count]  = 0xFFFF;
            htable->major[count]  = 0x0000;
        };

        code <<= 1;
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_decoder_read_qtable(BYTE *qtable)
{
    BYTE i;

    for (i = 0; i < 64; i++)
    {
        GET_BYTE();
        qtable[i] = data_byte;
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_decoder_process_scan(void)
{
    CMP *_cmp;
    DWORD img_x,       img_y;
    WORD img_offset_x, img_offset_y;
    WORD mcu_x,        mcu_y;
    BYTE i;
    BYTE du_x,         du_y;
    BYTE x1,           y1;
    BYTE _x1,          _y1;
    BYTE               __y1;
    BYTE x2,           y2;

    //process scan data
    for (mcu_y = 0; mcu_y < img_height; mcu_y += mcu_height) //mcu vertical
    {
        for (mcu_x = 0; mcu_x < img_width; mcu_x += mcu_width) //mcu horizontal
        {
            for (i = 0; i < scan.cmp_count; i++) //component
            {
                _cmp = scan.cmp[i];

                for (du_y = 0; du_y < _cmp->height; du_y += 8) //data unit vertical
                {
                    img_offset_y = mcu_y + du_y;

                    for (du_x = 0; du_x < _cmp->width; du_x += 8) //data unit horizontal
                    {
                        img_offset_x = mcu_x + du_x;

                        //decoding
                        jpeg_decoder_get_data_unit(_cmp->index);
                        jpeg_decoder_dequantize(_cmp->index);
                        jpeg_decoder_idct();

                        //subsampling
                        for (y1 = 0, _y1 = 0, __y1 = 0; y1 < 8; y1++, __y1 += 8)
                        {
                            for (y2 = 0; y2 < _cmp->scale_y; y2++)
                            {
                                img_y = img_offset_y + _y1 + y2;

                                if (img_y < img_height)
                                {
                                    img_y *= buffer.width;

                                    for (x1 = 0, _x1 = 0; x1 < 8; x1++)
                                    {
                                        for (x2 = 0; x2 < _cmp->scale_x; x2++)
                                        {
                                            img_x = img_offset_x + _x1 + x2;

                                            if (img_x < img_width)
                                                buffer.cmp[_cmp->index][img_x + img_y] = data_unit[x1 + __y1];
                                        };

                                        _x1 += _cmp->scale_x;
                                    };
                                };
                            };

                            _y1 += _cmp->scale_y;
                        };
                    }; //data unit horizontal
                }; //data unit vertical
            }; //component
        }; //mcu horizontal
    }; //mcu vertical
};
//-----------------------------------------------------------------------------------------------------
WORD jpeg_decoder_peek_bit(BYTE count)
{
    BYTE _data_byte;

    flag_restart = 0;

    while (bitstring.count < count && ! END_OF_DATA)
    {
        GET_BYTE();
        _data_byte = data_byte;

        if (data_byte == 0xFF)
        {
            while (data_byte == 0xFF) GET_BYTE();

            if (data_byte)
            {
                bitstring.value = 0;
                bitstring.count = 0;

                if (data_byte >= 0xD0 && data_byte <= 0xD7) //RST marker
                {
                    flag_restart   = 1;
                }
                else
                {
                    flag_marker    = 1;
                    data_position -= 2;
                };

                return 0;
            };
        };

        bitstring.value <<= 8;
        bitstring.value  |= _data_byte;
        bitstring.count  += 8;
    };

    return bitstring.value >> (bitstring.count - count);
};
//-----------------------------------------------------------------------------------------------------
WORD jpeg_decoder_get_bit(BYTE count)
{
    WORD value = jpeg_decoder_peek_bit(count);
    jpeg_decoder_clear_bit(count);

    return value;
};
//-----------------------------------------------------------------------------------------------------
void jpeg_decoder_clear_bit(BYTE count)
{
    if (count > bitstring.count) bitstring.count  = 0;
    else                         bitstring.count -= count;
    bitstring.value &= bitmask1[bitstring.count];
};
//-----------------------------------------------------------------------------------------------------
BYTE jpeg_decoder_get_hvalue(HTABLE *htable)
{
    WORD code;
    BYTE i;
    BYTE count;

    i = htable->min_count;

    while (i <= 16)
    {
        count = i - 1;
        code  = jpeg_decoder_peek_bit(i);

        if (flag_marker) return 0;
        if (flag_restart)
        {
            for (i = 0; i < cmp_count; i++) cmp[i].dc = 0;
            scan.eobrun  = 0;
            i            = htable->min_count;
            continue;
        };

        if (code >= htable->minor[count] && code <= htable->major[count])
        {
            jpeg_decoder_clear_bit(i);
            code        -= htable->minor[count];
            return htable->value[WORD_HIGH_LOW(count, code)];
        };

        i++;
    };

    ERROR("invalid huffman code");
    return 0;
};
//-----------------------------------------------------------------------------------------------------
void jpeg_decoder_get_data_unit(BYTE index)
{
    SWORD value;
    BYTE zero;
    BYTE count;
    BYTE i;
    BYTE c;

    //DC coefficient
    if (count = jpeg_decoder_get_hvalue(cmp[index].htable_dc))
    {
        value         = jpeg_decoder_get_bit(count);
        data_unit[0]  = cmp[index].dc + CONVERT_SWORD(value, count);
        cmp[index].dc = data_unit[0];
    }
    else
    {
        data_unit[0]  = cmp[index].dc;
    };

    //AC coefficient
    i = 1;

    while (i < 64)
    {
        value = jpeg_decoder_get_hvalue(cmp[index].htable_ac);
        count = BYTE_LOW(value);
        zero  = BYTE_HIGH(value);

        if (count == 0)
        {
            if (zero == 0)       break;  //EOB
            else if (zero == 15) zero++; //16 zero marker
        };

        for (c = i + zero; i < c; i++) data_unit[i] = 0;

        if (count)
        {
            value          = jpeg_decoder_get_bit(count);
            data_unit[i++] = CONVERT_SWORD(value, count);
        };
    };

    for (; i < 64; i++) data_unit[i] = 0;
};
//-----------------------------------------------------------------------------------------------------
void jpeg_decoder_dequantize(BYTE index)
{
        SWORD _data_unit[64];
        BYTE i;

        for (i = 0; i < 64; i++) _data_unit[i] = data_unit[i] * cmp[index].qtable[i];
        for (i = 0; i < 64; i++) data_unit[i]  = _data_unit[stable[i]];
};
//-----------------------------------------------------------------------------------------------------
void jpeg_decoder_idct(void)
{
    SDWORD a0, a1, a2, a3, a4, a5, a6, a7, a8;
    SDWORD b0, b1, b2, b3, b4, b5, b6, b7, b8;
    SWORD *p;
    BYTE i;

    //process row
    p = data_unit;

    for (i = 0; i < 8; i++)
    {
        if ((p[0] | p[1] | p[2] | p[3] | p[4] | p[5] | p[6] | p[7]) == 0)
        {
            p += 8;
            continue;
        };

        a0     = (p[0] << 11) + 128;
        a1     =  p[1];
        a2     =  p[2];
        a3     =  p[3];
        a4     =  p[4] << 11;
        a5     =  p[5];
        a6     =  p[6];
        a7     =  p[7];

        b4     = 1108 * (a6 + a2);

        b8     = a0 + a4;
        b0     = a0 - a4;

        b6     = b4 - 3784 * a6;
        b2     = b4 + 1568 * a2;

        b3     = b8 + b2;
        b8    -= b2;

        b2     = b0 + b6;
        b0    -= b6;

        a8     =  565 * (a1 + a7);
        b7     = a8 - 3406 * a7;
        b1     = a8 + 2276 * a1;

        a8     = 2408 * (a3 + a5);
        a3     = a8 - 4017 * a3;
        b5     = a8 -  799 * a5;

        b4     = b1 + b5;
        b1    -= b5;

        b5     = b7 + a3;
        b7    -= a3;

        b6     = (181 * (b1 + b7) + 128) >> 8;
        b1     = (181 * (b1 - b7) + 128) >> 8;

        p[0]   = (b3 + b4) >> 8;
        p[1]   = (b2 + b6) >> 8;
        p[2]   = (b0 + b1) >> 8;
        p[3]   = (b8 + b5) >> 8;
        p[4]   = (b8 - b5) >> 8;
        p[5]   = (b0 - b1) >> 8;
        p[6]   = (b2 - b6) >> 8;
        p[7]   = (b3 - b4) >> 8;

        p     += 8;
    };

    //process column
    p = data_unit;

    for (i = 0; i < 8; i++)
    {
        if ((p[0] | p[8] | p[16] | p[24] | p[32] | p[40] | p[48] | p[56]) == 0)
        {
            p += 1;
            continue;
        };

        a0     = (p[ 0] << 8) + 8192;
        a1     =  p[ 8];
        a2     =  p[16];
        a3     =  p[24];
        a4     =  p[32] << 8;
        a5     =  p[40];
        a6     =  p[48];
        a7     =  p[56];

        b4     = 1108 * (a6 + a2) + 4;

        b8     = a0 + a4;
        b0     = a0 - a4;

        b6     = (b4 - 3784 * a6) >> 3;
        b2     = (b4 + 1568 * a2) >> 3;

        b3     = b8 + b2;
        b8    -= b2;

        b2     = b0 + b6;
        b0    -= b6;

        a8     =  565 * (a1 + a7) + 4;
        b7     = (a8 - 3406 * a7) >> 3;
        b1     = (a8 + 2276 * a1) >> 3;

        a8     = 2408 * (a3 + a5) + 4;
        a3     = (a8 - 4017 * a3) >> 3;
        b5     = (a8 -  799 * a5) >> 3;

        b4     = b1 + b5;
        b1    -= b5;

        b5     = b7 + a3;
        b7    -= a3;

        b6     = (181 * (b1 + b7) + 128) >> 8;
        b1     = (181 * (b1 - b7) + 128) >> 8;

        p[ 0]  = (b3 + b4) >> 14;
        p[ 8]  = (b2 + b6) >> 14;
        p[16]  = (b0 + b1) >> 14;
        p[24]  = (b8 + b5) >> 14;
        p[32]  = (b8 - b5) >> 14;
        p[40]  = (b0 - b1) >> 14;
        p[48]  = (b2 - b6) >> 14;
        p[56]  = (b3 - b4) >> 14;

        p     += 1;
    };

    for (i = 0; i < 64; i++)
    {
                data_unit[i] += 128;

        if (data_unit[i] < 0)        data_unit[i] = 0;
        else if (data_unit[i] > 255) data_unit[i] = 255;
    };
};

//-----------------------------------------------------------------------------------------------------
//===< PROGRESSIVE >===================================================================================
//-----------------------------------------------------------------------------------------------------

void jpeg_pdecoder_process_scan(void)
{
    CMP *_cmp;
    DWORD position;
    SWORD *_buffer;
    WORD img_offset_x, img_offset_y;
    void (*get_data_unit)(BYTE, SWORD *);

    //decoding
    if (scan.ss_start) //AC
    {
        if (scan.sa_high) get_data_unit = jpeg_pdecoder_ac_approximation;
        else              get_data_unit = jpeg_pdecoder_ac_base;
    }
    else //DC
    {
        if (scan.sa_high) get_data_unit = jpeg_pdecoder_dc_approximation;
        else              get_data_unit = jpeg_pdecoder_dc_base;
    };

    //process scan data
    if (scan.cmp_count == 1 && scan.ss_start)
    {
        BYTE width  = scan.cmp[0]->scale_x * 8;
        BYTE height = scan.cmp[0]->scale_y * 8;

        _cmp        = scan.cmp[0];

        for (img_offset_y = 0; img_offset_y < img_height; img_offset_y += height)
        {
            for (img_offset_x = 0; img_offset_x < img_width; img_offset_x += width)
            {
                position = img_offset_x + img_offset_y * buffer.width;
                _buffer  = &buffer.cmp[_cmp->index][position];

                get_data_unit(_cmp->index, _buffer);

                if (flag_marker) return;
            };
        };
    }
    else
    {
        WORD mcu_x, mcu_y;
        BYTE du_x,  du_y;
        BYTE i;

        for (mcu_y = 0; mcu_y < img_height; mcu_y += mcu_height) //mcu vertical
        {
            for (mcu_x = 0; mcu_x < img_width; mcu_x += mcu_width) //mcu horizontal
            {
                for (i = 0; i < scan.cmp_count; i++) //component
                {
                    _cmp = scan.cmp[i];

                    for (du_y = 0; du_y < _cmp->height; du_y += 8) //data unit vertical
                    {
                        img_offset_y = mcu_y + du_y;

                        for (du_x = 0; du_x < _cmp->width; du_x += 8) //data unit horizontal
                        {
                            img_offset_x = mcu_x + du_x;
                            position     = img_offset_x + img_offset_y * buffer.width;
                            _buffer      = &buffer.cmp[_cmp->index][position];

                            get_data_unit(_cmp->index, _buffer);

                            if (flag_marker) return;
                        }; //data unit horizontal
                    }; //data unit vertical
                }; //component
            }; //mcu horizontal
        }; //mcu vertical
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_pdecoder_process_data(void)
{
    CMP *_cmp;
    DWORD position;
    DWORD img_x,       img_y;
    SWORD *_buffer;
    WORD img_offset_x, img_offset_y;
    WORD mcu_x,        mcu_y;
    BYTE du_x,         du_y;
    BYTE i;
    BYTE x1,           y1;
    BYTE _x1,          _y1;
    BYTE               __y1;
    BYTE x2,           y2;

    //process data buffer
    for (mcu_y = 0; mcu_y < img_height; mcu_y += mcu_height) //mcu vertical
    {
        for (mcu_x = 0; mcu_x < img_width; mcu_x += mcu_width) //mcu horizontal
        {
            for (i = 0; i < cmp_count; i++) //component
            {
                _cmp = &cmp[i];

                for (du_y = 0; du_y < _cmp->height; du_y += 8) //data unit vertical
                {
                    img_offset_y = mcu_y + du_y;

                    for (du_x = 0; du_x < _cmp->width; du_x += 8) //data unit horizontal
                    {
                        img_offset_x = mcu_x + du_x;
                        position     = img_offset_x + img_offset_y * buffer.width;
                        _buffer      = &buffer.cmp[_cmp->index][position];

                        //decoding
                        jpeg_pdecoder_dequantize(_cmp->index, _buffer);
                        jpeg_decoder_idct();

                        //subsampling
                        for (y1 = 0, _y1 = 0, __y1 = 0; y1 < 8; y1++, __y1 += 8)
                        {
                            for (y2 = 0; y2 < _cmp->scale_y; y2++)
                            {
                                img_y = img_offset_y + _y1 + y2;

                                if (img_y < img_height)
                                {
                                    img_y *= buffer.width;

                                    for (x1 = 0, _x1 = 0; x1 < 8; x1++)
                                    {
                                        for (x2 = 0; x2 < _cmp->scale_x; x2++)
                                        {
                                            img_x = img_offset_x + _x1 + x2;

                                            if (img_x < img_width)
                                                buffer.cmp[_cmp->index][img_x + img_y] = data_unit[x1 + __y1];
                                        };

                                        _x1 += _cmp->scale_x;
                                    };
                                };
                            };

                            _y1 += _cmp->scale_y;
                        };
                    }; //data unit horizontal
                }; //data unit vertical
            }; //component
        }; //mcu horizontal
    }; //mcu vertical
};
//-----------------------------------------------------------------------------------------------------
void jpeg_pdecoder_dc_base(BYTE index, SWORD *buffer)
{
    SWORD value;
    BYTE count;

    if (count = jpeg_decoder_get_hvalue(cmp[index].htable_dc))
    {
        value         = jpeg_decoder_get_bit(count);
        value         = CONVERT_SWORD(value, count);
        cmp[index].dc = value += cmp[index].dc;
        buffer[0]     = value << scan.sa_low;
    }
    else
    {
        buffer[0]     = cmp[index].dc << scan.sa_low;
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_pdecoder_dc_approximation(BYTE index, SWORD *buffer)
{
    if (jpeg_decoder_get_bit(1)) buffer[0] += 1 << scan.sa_low;
};
//-----------------------------------------------------------------------------------------------------
void jpeg_pdecoder_ac_base(BYTE index, SWORD *buffer)
{
    SDWORD value;
    BYTE zero = 0;
    BYTE count;
    BYTE i;

    if (scan.eobrun)
    {
        scan.eobrun--;
        return;
    };

    for (i = scan.ss_start; i <= scan.ss_end; i++)
    {
        value  = jpeg_decoder_get_hvalue(cmp[index].htable_ac);
        zero   = BYTE_HIGH(value);
        count  = BYTE_LOW(value);

        if (count)
        {
            i                          += zero;
            value                       = jpeg_decoder_get_bit(count);
            value                       = CONVERT_SWORD(value, count);
            buffer[buffer_position[i]]  = value << scan.sa_low;
        }
        else if (zero == 15)
        {
            i                          += 15;
        }
        else
        {
            scan.eobrun                 = 1 << zero;
            if (zero) scan.eobrun += jpeg_decoder_get_bit(zero);
            scan.eobrun--;
            break;
        };
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_pdecoder_ac_approximation(BYTE index, SWORD *buffer)
{
    SWORD value;
    SWORD positive =  1 << scan.sa_low;
    SWORD negative = -1 << scan.sa_low;
    SWORD *coefficient;
    BYTE zero;
    BYTE count;
    BYTE i         = scan.ss_start;

    if (! scan.eobrun)
    {
        for (; i <= scan.ss_end; i++)
        {
            value = jpeg_decoder_get_hvalue(cmp[index].htable_ac);
            zero  = BYTE_HIGH(value);
            count = BYTE_LOW(value);

            if (count)
            {
                value       = jpeg_decoder_get_bit(1) ? positive : negative;
            }
            else if (zero != 15)
            {
                scan.eobrun = 1 << zero;
                if (zero) scan.eobrun += jpeg_decoder_get_bit(zero);
                break;
            };

            do
            {
                coefficient = &buffer[buffer_position[i]];

                if (*coefficient)
                {
                    if (jpeg_decoder_get_bit(1))
                    {
                        if (! (*coefficient & positive))
                        {
                            *coefficient += (*coefficient > 0) ? positive : negative;
                        };
                    };
                }
                else
                {
                    if (! zero) break;
                    zero--;
                };

                i++;
            }
            while (i <= scan.ss_end);

            if (count) buffer[buffer_position[i]] = value;
        };
    };

    if (scan.eobrun)
    {
        for (; i <= scan.ss_end; i++)
        {
            coefficient = &buffer[buffer_position[i]];

            if (*coefficient)
            {
                if (jpeg_decoder_get_bit(1))
                {
                    if (! (*coefficient & positive))
                    {
                        *coefficient += (*coefficient > 0) ? positive : negative;
                    };
                };
            };
        };

        scan.eobrun--;
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_pdecoder_dequantize(BYTE index, SWORD *buffer)
{
    BYTE i;

    for (i = 0; i < 64; i++) data_unit[i] = buffer[buffer_position[stable[i]]] * cmp[index].qtable[stable[i]];
};

//-----------------------------------------------------------------------------------------------------
//===< RESAMPLER >=====================================================================================
//-----------------------------------------------------------------------------------------------------

int jpeg_resample(void)
{
    //downscale
    WORD img_width_half;
    WORD img_height_half;
    DWORD buffer_width_double;

    //scale
    double factor_x,     factor_y;
    double fraction_x,   fraction_y;
    double _fraction_x,  _fraction_y;
    double __fraction_x, __fraction_y;

    //buffer image
    DWORD source_x1,     source_y1;
    DWORD source_x2,     source_y2;
    DWORD source_x3,     source_y3;
    DWORD source_x4,     source_y4;

    //buffer line
    BYTE line1;
    BYTE line2;
    BYTE line3;
    BYTE line4;
    BYTE value;

    //target image
    WORD target_x,       target_y;
    DWORD _target,       _target_y;

    BYTE i;

    //downscale
    img_width_half      = img_width  / 2;
    img_height_half     = img_height / 2;
    buffer_width_double = buffer.width + buffer.width;

    while (target.width < img_width_half && target.height < img_height_half)
    {
        source_y1 = 0;
        source_y2 = buffer.width;

        for (target_y = 0; target_y < img_height_half; target_y++)
        {
            source_x1 = 0;
            source_x2 = 1;
            _target_y = target_y * buffer.width;

            for (target_x = 0; target_x < img_width_half; target_x++)
            {
                _target = target_x + _target_y;

                for (i = 0; i < cmp_count; i++)
                {
                    if (i == 0)
                    {
                        value = ((BYTE)buffer.cmp[i][source_x1 + source_y1] +
                                 (BYTE)buffer.cmp[i][source_x1 + source_y2] +
                                 (BYTE)buffer.cmp[i][source_x2 + source_y1] +
                                 (BYTE)buffer.cmp[i][source_x2 + source_y2]) / 4;
                    }
                    else
                    {
                        value = (BYTE)buffer.cmp[i][source_x1 + source_y1];
                    };

                    buffer.cmp[i][_target] = value;
                };

                source_x1 += 2;
                source_x2 += 2;
            };

            source_y1 += buffer_width_double;
            source_y2 += buffer_width_double;
        };

        img_width       = img_width_half;
        img_height      = img_height_half;
        img_width_half  = img_width  / 2;
        img_height_half = img_height / 2;
    };

    //target image size
    if (target.width == 0)       target.width  = target.height * img_width  / img_height;
    else if (target.height == 0) target.height = target.width  * img_height / img_width;

    //target buffer allocation
    target.size = target.width * target.height * sizeof(BYTE);

    for (i = 0; i < cmp_count; i++)
    {
        target.cmp[i] = (BYTE *)malloc(target.size);
        if (target.cmp[i] == NULL) ERROR("out of memory");
    };

    factor_x = (double)img_width  / (double)target.width;
    factor_y = (double)img_height / (double)target.height;

    //bicubic interpolation
    for (target_y = 0; target_y < target.height; target_y++)
    {
        fraction_y    = target_y * factor_y;

        source_y2     = (WORD)fraction_y;
        source_y1     = (source_y2     >  0)          ? source_y2  - 1 : 0;
        source_y3     = (source_y2 + 1 >= img_height) ? img_height - 1 : source_y2 + 1;
        source_y4     = (source_y2 + 2 >= img_height) ? img_height - 1 : source_y2 + 2;

        _fraction_y   = fraction_y - source_y2;
        __fraction_y  = 1.0 - _fraction_y;

        source_y1    *= buffer.width;
        source_y2    *= buffer.width;
        source_y3    *= buffer.width;
        source_y4    *= buffer.width;
        _target_y     = target_y * target.width;

        for (target_x = 0; target_x < target.width; target_x++)
        {
            fraction_x   = target_x * factor_x;

            source_x2    = (WORD)fraction_x;
            source_x1    = (source_x2     >  0)         ? source_x2 - 1 : 0;
            source_x3    = (source_x2 + 1 >= img_width) ? img_width - 1 : source_x2 + 1;
            source_x4    = (source_x2 + 2 >= img_width) ? img_width - 1 : source_x2 + 2;
            _target      = target_x + _target_y;

            _fraction_x  = fraction_x - source_x2;
            __fraction_x = 1.0 - _fraction_x;

            for (i = 0; i < cmp_count; i++)
            {
                if (i == 0) //bicubic (luminance)
                {
                    line1 = jpeg_resampler_cubic((BYTE)buffer.cmp[i][source_x1 + source_y1],
                                                 (BYTE)buffer.cmp[i][source_x2 + source_y1],
                                                 (BYTE)buffer.cmp[i][source_x3 + source_y1],
                                                 (BYTE)buffer.cmp[i][source_x4 + source_y1],
                                                 _fraction_x);
                    line2 = jpeg_resampler_cubic((BYTE)buffer.cmp[i][source_x1 + source_y2],
                                                 (BYTE)buffer.cmp[i][source_x2 + source_y2],
                                                 (BYTE)buffer.cmp[i][source_x3 + source_y2],
                                                 (BYTE)buffer.cmp[i][source_x4 + source_y2],
                                                 _fraction_x);
                    line3 = jpeg_resampler_cubic((BYTE)buffer.cmp[i][source_x1 + source_y3],
                                                 (BYTE)buffer.cmp[i][source_x2 + source_y3],
                                                 (BYTE)buffer.cmp[i][source_x3 + source_y3],
                                                 (BYTE)buffer.cmp[i][source_x4 + source_y3],
                                                 _fraction_x);
                    line4 = jpeg_resampler_cubic((BYTE)buffer.cmp[i][source_x1 + source_y4],
                                                 (BYTE)buffer.cmp[i][source_x2 + source_y4],
                                                 (BYTE)buffer.cmp[i][source_x3 + source_y4],
                                                 (BYTE)buffer.cmp[i][source_x4 + source_y4],
                                                 _fraction_x);
                    value = jpeg_resampler_cubic(line1, line2, line3, line4, _fraction_y);
                }
                else //bilinear (chrominance)
                {
                    line1 = (BYTE)(buffer.cmp[i][source_x1 + source_y1] * __fraction_x +
                                   buffer.cmp[i][source_x2 + source_y1] * _fraction_x);
                    line2 = (BYTE)(buffer.cmp[i][source_x1 + source_y2] * __fraction_x +
                                   buffer.cmp[i][source_x2 + source_y2] * _fraction_x);
                    value = (BYTE)(line1 * __fraction_y + line2 * _fraction_y);
                };

                target.cmp[i][_target] = value;
            };
        };
    };

    return 1;
};
//-----------------------------------------------------------------------------------------------------
BYTE jpeg_resampler_cubic(BYTE value1, BYTE value2, BYTE value3, BYTE value4, double fraction)
{
    double fraction_sqare = fraction * fraction;

    SWORD a = value1 - value2;
    SWORD b = value4 - value3 - a;
    SWORD c = a - b;
    SWORD d = value3 - value1;

    SWORD e = (SWORD)(b * fraction_sqare * fraction);
    SWORD f = (SWORD)(c * fraction_sqare);
    SWORD g = (SWORD)(d * fraction);

    SWORD i = e + f + g + value2;

    if (i < 0)        return   0;
    else if (i > 255) return 255;
    return (BYTE)i;
};

//-----------------------------------------------------------------------------------------------------
//===< ENCODER >=======================================================================================
//-----------------------------------------------------------------------------------------------------

int jpeg_encode(const char *file)
{
    //mcu
    WORD mcu_x,            mcu_y;
    BYTE mcu_width,        mcu_height;

    //image
    DWORD img_x,           img_y;
    DWORD _img_x,          _img_y;
    WORD img_offset_x,     img_offset_y;

    //component
    BYTE cmp_max_factor_x, cmp_max_factor_y;
    BYTE cmp_scale_x[3],   cmp_scale_y[3];
    BYTE cmp_width[3],     cmp_height[3];

    BYTE i;
    BYTE _i;
    BYTE x1,               y1;
    BYTE _x1,              _y1;
    BYTE                   __y1;
    BYTE x2,               y2;
    BYTE du_x,             du_y;
    BYTE du_pos;

#ifdef DEBUG
    file_handle = fopen("debug.jpg", "wb");
#else
    if (file != NULL)
    {
        file_handle = fopen(file, "wb");
        if (file_handle == NULL) ERROR("target file invalid");
    }
    else
    {
        file_handle = stdout;
    #ifdef SYS_WIN32
        _setmode(_fileno(file_handle), _O_BINARY);
    #endif
        fputs("HTTP/1.0 200 OK\n", file_handle);
        fputs("Server: nuos\n", file_handle);
        fputs("Content-Type: image/jpeg\n", file_handle);
        fputs("\n", file_handle);
    };
    fflush(file_handle);
#endif

    //reset bit buffer
    bitstring.count = 0;
    bitstring.value = 0;

    //adjust quality
    for (i = 0; i < 64; i++)
    {
        std_lqtable[i] = (BYTE)(std_lqtable[i] * QUALITY_LUMINANCE   / 100);
        if (std_lqtable[i] < 1) std_lqtable[i] = 1;
        std_cqtable[i] = (BYTE)(std_cqtable[i] * QUALITY_CHROMINANCE / 100);
        if (std_cqtable[i] < 1) std_cqtable[i] = 1;
    };

    //create huffman table
    jpeg_encoder_create_htable(std_dc_lhtable, std_dc_lhcount, std_dc_lhvalue);
    jpeg_encoder_create_htable(std_dc_chtable, std_dc_chcount, std_dc_chvalue);
    jpeg_encoder_create_htable(std_ac_lhtable, std_ac_lhcount, std_ac_lhvalue);
    jpeg_encoder_create_htable(std_ac_chtable, std_ac_chcount, std_ac_chvalue);

    //component information
    cmp[0].dc        = 0;
    cmp[0].qtable    = std_lqtable;
    cmp[0].htable_dc = std_dc_lhtable;
    cmp[0].htable_ac = std_ac_lhtable;

    for (i = 1; i < cmp_count; i++)
    {
        cmp[i].dc        = 0;
        cmp[i].qtable    = std_cqtable;
        cmp[i].htable_dc = std_dc_chtable;
        cmp[i].htable_ac = std_ac_chtable;
    };

    //SOI marker
    PUT_BYTE(0xFF); PUT_BYTE(SOI);
//.....................................................................................................
    //COM marker
    PUT_BYTE(0xFF); PUT_BYTE(COM);
    //length
    PUT_WORD(52);
    //comment
    for (i = 0; i < 50; i++) PUT_BYTE(comment[i]);
//.....................................................................................................
    //APP0 marker
    PUT_BYTE(0xFF); PUT_BYTE(APP0);
    //length
    PUT_WORD(14);
    //JFIF\0
    PUT_BYTE(0x4A); PUT_BYTE(0x46); PUT_BYTE(0x49); PUT_BYTE(0x46); PUT_BYTE(0x00);
    //version 1.0
    PUT_BYTE(0x01); PUT_BYTE(0x00);
    //aspect ratio (pixel)
    PUT_BYTE(0x00);
    //horizontal/vertical pixel density
    PUT_WORD(0x0001); PUT_WORD(0x0001);
    //thumbnail width/height
    PUT_BYTE(0x00); PUT_BYTE(0x00);
//.....................................................................................................
    //DHT marker
    PUT_BYTE(0xFF); PUT_BYTE(DHT);
    //length
    PUT_WORD((cmp_count > 1) ? 418 : 210);
    //dc luminance huffman table
    PUT_BYTE(0x00); //id
    for (i = 0; i <  16; i++) PUT_BYTE(std_dc_lhcount[i]);
    for (i = 0; i <  12; i++) PUT_BYTE(std_dc_lhvalue[i]);
    //ac luminance huffman table
    PUT_BYTE(0x10); //id
    for (i = 0; i <  16; i++) PUT_BYTE(std_ac_lhcount[i]);
    for (i = 0; i < 162; i++) PUT_BYTE(std_ac_lhvalue[i]);
    if (cmp_count > 1)
    {
        //dc chrominance huffman table
        PUT_BYTE(0x01); //id
        for (i = 0; i <  16; i++) PUT_BYTE(std_dc_chcount[i]);
        for (i = 0; i <  12; i++) PUT_BYTE(std_dc_chvalue[i]);
        //ac chrominance huffman table
        PUT_BYTE(0x11); //id
        for (i = 0; i <  16; i++) PUT_BYTE(std_ac_chcount[i]);
        for (i = 0; i < 162; i++) PUT_BYTE(std_ac_chvalue[i]);
    };
//.....................................................................................................
    //DQT marker
    PUT_BYTE(0xFF); PUT_BYTE(DQT);
    //length
    PUT_WORD((cmp_count > 1) ? 132 : 67);
    //luminance quantization table
    PUT_BYTE(0x00); //id
    for (i = 0; i < 64; i++) PUT_BYTE(std_lqtable[i]);
    if (cmp_count > 1)
    {
        //chrominance quantization table
        PUT_BYTE(0x01); //id
        for (i = 0; i < 64; i++) PUT_BYTE(std_cqtable[i]);
    };
//.....................................................................................................
    //SOF0 marker
    PUT_BYTE(0xFF); PUT_BYTE(SOF0);
    //length
    PUT_WORD((cmp_count > 1) ? 17 : 11);
    //precision
    PUT_BYTE(0x08);
    //height/width
    PUT_WORD(target.height); PUT_WORD(target.width);
    //component count
    PUT_BYTE(cmp_count);
    if (cmp_count > 1)
    {
        mcu_width        = 16; mcu_height       = 16;
        cmp_max_factor_x =  2; cmp_max_factor_y =  2;
        cmp_scale_x[0]   =  1; cmp_scale_y[0]   =  1;
        cmp_scale_x[1]   =  2; cmp_scale_y[1]   =  2;
        cmp_scale_x[2]   =  2; cmp_scale_y[2]   =  2;
        cmp_width[0]     = 16; cmp_height[0]    = 16;
        cmp_width[1]     =  8; cmp_height[1]    =  8;
        cmp_width[2]     =  8; cmp_height[2]    =  8;

        //Y component: id/factor/qtable id
        PUT_BYTE(0x01); PUT_BYTE(0x22); PUT_BYTE(0x00);
        //U component: id/factor/qtable id
        PUT_BYTE(0x02); PUT_BYTE(0x11); PUT_BYTE(0x01);
        //V component: id/factor/qtable id
        PUT_BYTE(0x03); PUT_BYTE(0x11); PUT_BYTE(0x01);
    }
    else
    {
        mcu_width        =  8; mcu_height       =  8;
        cmp_max_factor_x =  1; cmp_max_factor_y =  1;
        cmp_scale_x[0]   =  1; cmp_scale_y[0]   =  1;
        cmp_width[0]     =  8; cmp_height[0]    =  8;

        //Y component: id/factor/qtable id
        PUT_BYTE(0x01); PUT_BYTE(0x11); PUT_BYTE(0x00);
    };
//.....................................................................................................
    //SOS marker
    PUT_BYTE(0xFF); PUT_BYTE(SOS);
    //length
    PUT_WORD((cmp_count > 1) ? 12: 8);
    //component count
    PUT_BYTE(cmp_count);
    //Y component: id/htable id
    PUT_BYTE(0x01); PUT_BYTE(0x00);
    if (cmp_count > 1)
    {
        //U component: id/htable id
        PUT_BYTE(0x02); PUT_BYTE(0x11);
        //V component: id/htable id
        PUT_BYTE(0x03); PUT_BYTE(0x11);
    };
    //spectral selection start/end
    PUT_BYTE(0x00); PUT_BYTE(0x3F);
    //successive approximation high/low bit
    PUT_BYTE(0x00);

    //process scan data
    for (mcu_y = 0; mcu_y < target.height; mcu_y += mcu_height) //mcu vertical
    {
        for (mcu_x = 0; mcu_x < target.width; mcu_x += mcu_width) //mcu horizontal
        {
            for (i = 0; i < cmp_count; i++) //component
            {
                for (du_y = 0; du_y < cmp_height[i]; du_y += 8) //data unit vertical
                {
                    img_offset_y = mcu_y + du_y;

                    for (du_x = 0; du_x < cmp_width[i]; du_x += 8) //data unit horizontal
                    {
                        img_offset_x = mcu_x + du_x;

                        for (y1 = 0, _y1 = 0, __y1 = 0; y1 < 8; y1++, __y1 += 8)
                        {
                            img_y = img_offset_y + _y1;

                            for (x1 = 0, _x1 = 0; x1 < 8; x1++)
                            {
                                img_x              = img_offset_x + _x1;

                                du_pos             = x1 + __y1;
                                data_unit[du_pos]  = 0;
                                _i                 = 0;

                                //subsampling
                                for (y2 = 0; y2 < cmp_scale_y[i]; y2++)
                                {
                                    _img_y = img_y + y2;
                                    if (_img_y >= target.height) _img_y = target.height - 1;

                                    for (x2 = 0; x2 < cmp_scale_x[i]; x2++)
                                    {
                                        _img_x = img_x + x2;
                                        if (_img_x >= target.width) _img_x = target.width - 1;

                                        data_unit[du_pos] += target.cmp[i][_img_x + _img_y * target.width];
                                        _i++;
                                    };
                                };

                                data_unit[du_pos] /= _i;
                                _x1               += cmp_scale_x[i];
                            };

                            _y1 += cmp_scale_y[i];
                        };

                        //encoding
                        jpeg_encoder_fdct();
                        jpeg_encoder_quantize(i);
                        jpeg_encoder_put_data_unit(i);
                    }; //data unit horizontal
                }; //data unit vertical
            }; //component
        }; //mcu horizontal
    }; //mcu vertical

    if (bitstring.count) jpeg_encoder_put_bit(7, 0x7F);

    PUT_BYTE(0xFF); PUT_BYTE(EOI);

#ifdef DEBUG
    fclose(file_handle);
#else
    if (file != NULL) fclose(file_handle);
    else              fflush(file_handle);
#endif

    return 1;
};
//-----------------------------------------------------------------------------------------------------
void jpeg_encoder_create_htable(STD_HTABLE *htable, const BYTE *hcount, const BYTE *hvalue)
{
    WORD code     = 0;
    BYTE position = 0;
    BYTE count;
    BYTE i;

    for (count = 0; count < 16; count++)
    {
        for (i = 0; i < hcount[count]; i++)
        {
            htable[hvalue[position]].value = code;
            htable[hvalue[position]].count = count + 1;
            position++;
            code++;
        };

        code <<= 1;
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_encoder_put_bit(BYTE count, WORD value)
{
    DWORD buffer;

    buffer            = value & bitmask1[count];
    bitstring.count  += count;
    buffer          <<= 24 - bitstring.count;
    bitstring.value  |= buffer;

    while (bitstring.count >= 8)
    {
        buffer            = (bitstring.value >> 16) & 0xFF;

        PUT_BYTE(buffer);
        if (buffer == 0xFF) PUT_BYTE(0x00);

        bitstring.value <<= 8;
        bitstring.count  -= 8;
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_encoder_put_data_unit(BYTE index)
{
    STD_HTABLE *htable;
    SWORD dc;
    WORD buffer;
    BYTE count;
    BYTE zrl;
    BYTE i;

    //DC coefficient
    htable        = cmp[index].htable_dc;
    dc            = data_unit[0] - cmp[index].dc;
    cmp[index].dc = data_unit[0];

    //bit count
    count         = 0;
    buffer        = (dc < 0) ? -dc : dc;
    while (buffer)
    {
        count++;
        buffer >>= 1;
    };

    //emit bit count
    jpeg_encoder_put_bit(htable[count].count, htable[count].value);

    //emit bit value
    jpeg_encoder_put_bit(count, CONVERT_WORD(dc, count));

    //AC coefficient
    htable      = cmp[index].htable_ac;
    zrl         = 0;

    for (i = 1; i < 64; i++)
    {
        if (! data_unit[i])
        {
            zrl++;
        }
        else
        {
            while (zrl >= 16)
            {
                jpeg_encoder_put_bit(htable[0xF0].count, htable[0xF0].value);
                zrl -= 16;
            };

            //bit count
            buffer = (data_unit[i] < 0) ? -data_unit[i] : data_unit[i];
            count  = 1;
            while (buffer >>= 1) count++;

            //emit zrl/bit count huffman code
            buffer = BYTE_HIGH_LOW(zrl, count);
            jpeg_encoder_put_bit(htable[buffer].count, htable[buffer].value);

            //emit bit value
            jpeg_encoder_put_bit(count, CONVERT_WORD(data_unit[i], count));

            zrl    = 0;
        };
    };

    if (zrl) jpeg_encoder_put_bit(htable[0x00].count, htable[0x00].value); //EOB
};
//-----------------------------------------------------------------------------------------------------
void jpeg_encoder_fdct(void)
{
    SDWORD a0, a1, a2, a3, a4, a5, a6, a7, a8;
    SWORD *p;
    BYTE i;

    for (i = 0; i < 64; i++) data_unit[i] -= 128;

    //process row
    p = data_unit;

    for (i = 0; i < 8; i++)
    {
        a8     = p[0] + p[7];
        a0     = p[0] - p[7];
        a7     = p[1] + p[6];
        a1     = p[1] - p[6];
        a6     = p[2] + p[5];
        a2     = p[2] - p[5];
        a5     = p[3] + p[4];
        a3     = p[3] - p[4];
        a4     = a8 + a5;
        a8    -= a5;
        a5     = a7 + a6;
        a7    -= a6;

        p[0]   = a4 + a5;
        p[4]   = a4 - a5;

        p[2]   = (a8 * 1338 + a7 *  554) >> 10;
        p[6]   = (a8 *  554 - a7 * 1338) >> 10;

        p[7]   = (a0 *  283 - a1 *  805 + a2 * 1204 - a3 * 1420) >> 10;
        p[5]   = (a0 *  805 - a1 * 1420 + a2 *  283 + a3 * 1204) >> 10;
        p[3]   = (a0 * 1204 - a1 *  283 - a2 * 1420 - a3 *  805) >> 10;
        p[1]   = (a0 * 1420 + a1 * 1204 + a2 *  805 + a3 *  283) >> 10;

        p     += 8;
    };

    //process column
    p = data_unit;

    for (i = 0; i < 8; i++)
    {
        a8     = p[ 0] + p[56];
        a0     = p[ 0] - p[56];
        a7     = p[ 8] + p[48];
        a1     = p[ 8] - p[48];
        a6     = p[16] + p[40];
        a2     = p[16] - p[40];
        a5     = p[24] + p[32];
        a3     = p[24] - p[32];
        a4     = a8 + a5;
        a8    -= a5;
        a5     = a7 + a6;
        a7    -= a6;

        p[ 0]  = (a4 + a5) >> 3;
        p[32]  = (a4 - a5) >> 3;

        p[16]  = (a8 * 1338 + a7 *  554) >> 13;
        p[48]  = (a8 *  554 - a7 * 1338) >> 13;

        p[56]  = (a0 *  283 - a1 *  805 + a2 * 1204 - a3 * 1420) >> 13;
        p[40]  = (a0 *  805 - a1 * 1420 + a2 *  283 + a3 * 1204) >> 13;
        p[24]  = (a0 * 1204 - a1 *  283 - a2 * 1420 - a3 *  805) >> 13;
        p[ 8]  = (a0 * 1420 + a1 * 1204 + a2 *  805 + a3 *  283) >> 13;

        p     += 1;
    };
};
//-----------------------------------------------------------------------------------------------------
void jpeg_encoder_quantize(BYTE index)
{
    BYTE i;
    SWORD _data_unit[64];
    BYTE threshold = (index == 0) ? THRESHOLD_LUMINANCE : THRESHOLD_CHROMINANCE;

    for (i = 0; i < 64; i++)
        _data_unit[i]        = (data_unit[i] >= -threshold && data_unit[i] <= threshold) ?
                               0 : data_unit[i] / cmp[index].qtable[stable[i]];
    for (i = 0; i < 64; i++)
        data_unit[stable[i]] =  _data_unit[i];
};
