<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function rc4encrypt($string,
                    $password)
{
        /*
         * encrypts string using password with rc4 algorithm
         *
         * binary safe, utf-8 compliant
         */

        if (! $password) return NULL;

        $string          = (string)$string;
        $key             = array();
        $state           = array();
        $return          = "";

        $string_length   = strlen($string);
        $password_length = strlen($password);

        for ($i = 0; $i < 256; $i++)
        {
                $key[$i]   = ord($password[$i % $password_length]);
                $state[$i] = $i;
        };

        for ($i = 0, $j = 0; $i < 256; $i++)
        {
                $j         = ($j + $state[$i] + $key[$i]) % 256;
                $k         = $state[$i];
                $state[$i] = $state[$j];
                $state[$j] = $k;
        };

        for ($i = 0, $j = 0, $k = 0; $i < $string_length; $i++)
        {
                $k          = ($k + 1)          % 256;
                $j          = ($j + $state[$k]) % 256;

                $l          = $state[$k];
                $state[$k]  = $state[$j];
                $state[$j]  = $l;

                $l          = $state[($state[$k] + $state[$j]) % 256];
                $return    .= chr(ord($string[$i]) ^ $l);
        };

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function rc4decrypt($string,
                    $password)
{
        /*
         * decrypts string using password with rc4 algorithm
         *
         * binary safe, utf-8 compliant
         */

        return rc4encrypt($string, $password);
};
//-----------------------------------------------------------------------------------------------------
function encchr($string)
{
        /*
         * encodes delimeters used by data class
         *
         * binary safe, utf-8 compliant
         */

        return strtr($string,
                     array("\x00" => "[chr0]",
                           "\n"   => "[chr10]",
                           "\r"   => "[chr13]",
                           ";"    => "[chr59]",
                           "="    => "[chr61]",
                           "["    => "[chr91]",
                           "\\"   => "[chr92]"));
};
//-----------------------------------------------------------------------------------------------------
function decchr($string)
{
        /*
         * decodes delimeters used by data class
         *
         * binary safe, utf-8 compliant
         */

        return strtr($string,
                     array("[chr0]"  => "\x00",
                           "[chr10]" => "\n",
                           "[chr13]" => "\r",
                           "[chr59]" => ";",
                           "[chr61]" => "=",
                           "[chr91]" => "[",
                           "[chr92]" => "\\"));
};
//-----------------------------------------------------------------------------------------------------
function sqlesc($value)
{
        /*
         * escapes functional SQL characters
         *
         * binary safe, utf-8 compliant
         */

        if (is_array($value))
        {
                while (list($key, $_value) = each($value))
                        $value[$key] = sqlesc($_value);

                return $value;
        };

        return strtr($value,
                     array("\x00" => "\\x00",
                           "\n"   => "\\n",
                           "\r"   => "\\r",
                           "\\"   => "\\\\",
                           "'"    => "\\'",
                           "\""   => "\\\"",
                           "\x1A" => "\\x1A"));
};
//-----------------------------------------------------------------------------------------------------
function q($string)
{
        /*
         * short alias for addslashes
         *
         * binary safe, utf-8 compliant
         */

        return str_replace("</", "<\/", addslashes($string));
};
//-----------------------------------------------------------------------------------------------------
function r($string)
{
        /*
         * short alias for rawurlencode
         *
         * binary safe, utf-8 compliant
         */

        return rawurlencode($string);
};
//-----------------------------------------------------------------------------------------------------
function x($string)
{
        /*
         * short alias for xmlspecialchars
         *
         * binary safe, utf-8 compliant
         */

        return xmlspecialchars($string);
};
//-----------------------------------------------------------------------------------------------------
function rq($string)
{
        /*
         * combines rawurlencode and addslashes
         *
         * binary safe, utf-8 compliant
         */

        return addslashes(rawurlencode($string));
};
//-----------------------------------------------------------------------------------------------------
function qr($string)
{
        /*
         * combines addslashes and rawurlencode
         *
         * binary safe, utf-8 compliant
         */

        return rawurlencode(addslashes($string));
};
//-----------------------------------------------------------------------------------------------------
function qx($string)
{
        /*
         * combines addslashes and xmlspecialchars
         *
         * binary safe, utf-8 compliant
         */

        return xmlspecialchars(addslashes($string));
};
//-----------------------------------------------------------------------------------------------------
function rx($string)
{
        /*
         * combines rawurlencode and xmlspecialchars
         *
         * binary safe, utf-8 compliant
         */

        return xmlspecialchars(rawurlencode($string));
};
//-----------------------------------------------------------------------------------------------------
function rqx($string)
{
        /*
         * combines rawurlencode, addslashes and xmlspecialchars
         *
         * binary safe, utf-8 compliant
         */

        return xmlspecialchars(addslashes(rawurlencode($string)));
};
//-----------------------------------------------------------------------------------------------------
function xmlspecialchars($string,
                         $decode = NULL)
{
        /*
         * encodes functional XML characters
         *
         * binary safe, utf-8 compliant
         */

        $translation = $decode ?
                       array("&quot;" => "\"",
                             "&amp;"  => "&",
                             "&lt;"   => "<",
                             "&gt;"   => ">") :
                       array("\""     => "&quot;",
                             "&"      => "&amp;",
                             "<"      => "&lt;",
                             ">"      => "&gt;");

        return strtr($string, $translation);
};
//-----------------------------------------------------------------------------------------------------
function htmlentities_decode($string)
{
        /*
         * decodes HTML entities
         *
         * binary safe, utf-8 compliant
         */

        static $translation;

        //replace numeric entities
        $string = preg_replace_callback(
                "/&#x([0-9a-f]+);/i",
                function($value) { return utf8_chr(hexdec($value[1])); },
                $string);
        $string = preg_replace_callback(
                "/&#([0-9]+);/",
                function($value) { return utf8_chr($value[1]); },
                $string);

        //replace literal entities
        if (empty($translation))
        {
                $translation  = array();
                $_translation = get_html_translation_table(HTML_ENTITIES);

                reset($_translation);
                while (list($key, $value) = each($_translation))
                        $translation[$value] = utf8_encode($key);
        };

        return strtr($string, $translation);
};
//-----------------------------------------------------------------------------------------------------
if (version_compare(PHP_VERSION, "5.3.0", "<"))
{
function quoted_printable_encode($string)
{
        /*
         * encodes string in quoted printable format
         *
         * binary safe
         */

        $string = preg_replace_callback(
        //                                !"#  $    =    ?    @    [\]   ^    `    {|}~
                "/[\\x00-\\x08\\x0A-\\x1F\\x20-\\x24\\x3D\\x3F\\x40\\x5B-\\x5E\\x60\\x7B-\\xFF]/",
                function($value) { return sprintf("=%02X", ord($value[0])); },
                $string);

        preg_match_all("/.{1,72}([^=]{0,3})?/", $string, $_string);
        return implode("=\r\n", $_string[0]);
};
};
//-----------------------------------------------------------------------------------------------------
function escape($string)
{
        /*
         * analogue implementation of javascript function
         *
         * binary safe, utf-8 compliant
         */

        return preg_replace("/(.)/",
                            function($value) { return "%" . bin2hex($value[1]); },
                            $string);
};
//-----------------------------------------------------------------------------------------------------
function strtonum($string)
{
        /*
         * converts string to number
         *
         * binary safe, utf-8 compliant
         */

        $search = "/(-?[0-9" . preg_quote(CMS_L_THOUSAND_SEPARATOR . CMS_L_DECIMAL_SEPARATOR, "/") . "]+)/iu";
        if (! preg_match($search, $string, $string)) return FALSE;

        $string = $string[0];
        $string = str_replace(CMS_L_THOUSAND_SEPARATOR, "", $string);
        $string = str_replace(CMS_L_DECIMAL_SEPARATOR, ".", $string);

        return floatval($string);
};
?>