<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function fingerprint($string,
                     $guarantee_threshold = 8,
                     $noise_threshold     = 5)
{
        /*
         * creates a fingerprint of string using binary karp-rabin
         * in combination with robust winnowing algorithm
         *
         * returns array of selected hashes
         *
         * basically this algorithm hashes each part of a string and
         * collects all local minima within each window of the hashes
         *
         * binary safe, utf-8 compliant
         */

        //hash algorithm constants
        $table = 65537;
        $base  = 257;

        //adjust parameters
        if ($noise_threshold > $guarantee_threshold)
                $noise_threshold = $guarantee_threshold;

        $window_size = $guarantee_threshold - $noise_threshold + 1;
        $kgram_size  = $noise_threshold;

        $c  = strlen($string);
        if ($c < $kgram_size)  $kgram_size  = $c;
        $c -= $kgram_size;
        if ($c < $window_size) $window_size = $c;

        //compute constant base ^ (kgram_size - 1)
        $const1 = 1;
        for ($i = 1; $i < $kgram_size; $i++)
                $const1 = ($const1 * $base) % $table;

        //compute constant base * table
        $const2 = $base * $table;

        //compute initial hash
        $hash = ord($string[0]);
        for ($i = 1; $i < $kgram_size; $i++)
               $hash = ($hash * $base + ord($string[$i])) % $table;

        //text too short for winnowing => return initial hash
        if ($window_size === 0)
                return array(0 => $hash);

        //fill initial window
        $window = array(0 => $hash);
        for ($i = 0; $i < $window_size; $i++)
        {
                $hash = ($hash - ord($string[$i]) * $const1 + $const2) % $table;
                $hash = ($hash * $base + ord($string[$i + $kgram_size])) % $table;

                $window[] = $hash;
        };

        $pos    = $window_size - 1;
        $min    = $pos;
        $result = array();

        while (TRUE)
        {
                //next window
                if ($min === $pos)
                {
                        //scan back for rightmost minimum in previous window
                        $_pos = $pos;

                        do
                        {
                                if (--$_pos < 0)                    $_pos += $window_size;
                                if ($window[$_pos] < $window[$min]) $min   = $_pos;
                        }
                        while ($_pos != $pos);

                        $result[] = $window[$min];
                }

                //current window
                else
                {
                        if ($window[$pos] < $window[$min])
                        {
                                $min      = $pos;
                                $result[] = $window[$min];
                        };
                };

                if ($i >= $c)                break;
                if (++$pos === $window_size) $pos = 0;

                //compute next hash
                $hash = ($hash - ord($string[$i]) * $const1 + $const2) % $table;
                $hash = ($hash * $base + ord($string[$i + $kgram_size])) % $table;

                $window[$pos] = $hash;

                $i++;
        };

        return $result;
};
//-----------------------------------------------------------------------------------------------------
function hmac_md5($string,
                  $key)
{
        /*
         * creates hash of string using key
         *
         * binary safe, utf-8 compliant
         */

        if (strlen($key) > 64) $key = pack("H*", md5($key));
        $key  = str_pad($key, 64, "\x00");

        $ipad = $key ^ str_repeat("\x36", 64);
        $opad = $key ^ str_repeat("\x5C", 64);

        return md5($opad . pack("H*", md5($ipad . $string)));
};
//-----------------------------------------------------------------------------------------------------
function simhash($string)
{
        /*
         * creates 64 bit similarity hash of string
         *
         * binary safe, utf-8 compliant
         */

        //initialze histogram
        $histogram = array_fill(0, 64, 0);

        //traverse string
        for ($i = 0, $c = strlen($string); $i < $c; $i++)
        {
                //generate shingle hash
                $hash  = md5(substr($string, $i, 4));
                $hash1 = hexdec(substr($hash, 0, 8));
                $hash2 = hexdec(substr($hash, 8, 8));

                //traverse hash bits
                for ($_i = 0; $_i < 32; $_i++)
                {
                        $bitmask              = 1 << $_i;
                        $histogram[$_i]      += (($hash1 & $bitmask) === 0) ? -1 : 1;
                        $histogram[$_i + 32] += (($hash2 & $bitmask) === 0) ? -1 : 1;
                };
        };

        //generate result hash
        $hash = "";
	    for ($i = 0; $i < 64; $i++) $hash .= ($histogram[$i] > 0) ? "1" : "0";

        return $hash;
};
?>