<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function ice_check_path($path)
{
        /*
         * check path for injected modifications
         *
         * binary safe, utf-8 compliant
         */

        if (preg_match("/(?:^|[\/\\\])\.+(?:[\/\\\]|$)/", $path)) exit();
};
//-----------------------------------------------------------------------------------------------------
function chgrp($filename,
               $group)
{
        /*
         * safer wrapper function for chgrp
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        return call_user_func_array("chgrp", $array);
};
//-----------------------------------------------------------------------------------------------------
function chmod($filename,
               $mode)
{
        /*
         * safer wrapper function for chmod
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        return call_user_func_array("chmod", $array);
};
//-----------------------------------------------------------------------------------------------------
function chown($filename,
               $user)
{
        /*
         * safer wrapper function for chown
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        return call_user_func_array("chown", $array);
};
//-----------------------------------------------------------------------------------------------------
function copy($source,
              $dest)
{
        /*
         * safer wrapper function for copy
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($source);
        ice_check_path($dest);

        $array = func_get_args();

        if (($return = call_user_func_array("copy", $array)) && is_file($dest))
                chmod($dest, is_file($source) ? fileperms($source) : (CMS_APACHE ? 0666 : 0644));

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function file_get_contents($filename)
{
        /*
         * safer wrapper function for file_get_contents
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        return call_user_func_array("file_get_contents", $array);
};
//-----------------------------------------------------------------------------------------------------
function file_put_contents($filename,
                           $data)
{
        /*
         * safer wrapper function for file_put_contents
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        $flag  = ! is_file($filename);

        if (($return = call_user_func_array("file_put_contents", $array)) && $flag && is_file($filename))
                chmod($filename, CMS_APACHE ? 0666 : 0644);

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function file($filename)
{
        /*
         * safer wrapper function for file
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        return call_user_func_array("file", $array);
};
//-----------------------------------------------------------------------------------------------------
function fopen($filename,
               $mode)
{
        /*
         * safer wrapper function for fopen
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        $flag  = ! is_file($filename);

        if (($return = call_user_func_array("fopen", $array)) && $flag && is_file($filename))
                chmod($filename, CMS_APACHE ? 0666 : 0644);

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function mkdir($pathname)
{
        /*
         * safer wrapper function for mkdir
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($pathname);

        $array      = func_get_args();
        $basename   = basename($pathname);
        $char       = substr($basename, 0, 1);
        $executable = $char === "!";
        $protected  = $executable || ($char === "#");
        $mode       =  CMS_APACHE ? 0777 :        //apache module
                      ($protected ? 0700 : 0755); //cgi
        $array[1]   = $mode;

        if (
             ($return = call_user_func_array("mkdir", $array))
             &&
             $protected
             &&
             ($hfile = fopen("$pathname/.htaccess", "wb"))
           )
        {
                fputs($hfile,
                      "Order Allow,Deny\n" .
                      "Deny from all");
                if ($executable)
                        fputs($hfile,
                              "\nOptions +ExecCGI");
                fclose($hfile);
        };

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function move_uploaded_file($filename,
                            $destination)
{
        /*
         * safer wrapper function for move_uploaded_file
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);
        ice_check_path($destination);

        $array = func_get_args();

        if ($return = call_user_func_array("move_uploaded_file", $array))
                chmod($destination, CMS_APACHE ? 0666 : 0644);

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function opendir($path)
{
        /*
         * safer wrapper function for opendir
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($path);

        $array = func_get_args();
        return call_user_func_array("opendir", $array);
};
//-----------------------------------------------------------------------------------------------------
function readfile($filename)
{
        /*
         * safer wrapper function for readfile
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        return call_user_func_array("readfile", $array);
};
//-----------------------------------------------------------------------------------------------------
function rename($oldname,
                $newname)
{
        /*
         * safer wrapper function for rename
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($oldname);
        ice_check_path($newname);

        $array = func_get_args();
        return call_user_func_array("rename", $array);
};
//-----------------------------------------------------------------------------------------------------
function rmdir($dirname)
{
        /*
         * safer wrapper function for rmdir
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($dirname);

        $array = func_get_args();
        return call_user_func_array("rmdir", $array);
};
//-----------------------------------------------------------------------------------------------------
function unlink($filename)
{
        /*
         * safer wrapper function for unlink
         *
         * binary safe, utf-8 compliant
         */

        ice_check_path($filename);

        $array = func_get_args();
        return call_user_func_array("unlink", $array);
};
?>