<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function image($url,
               $width  = NULL,
               $height = NULL,
               $title  = NULL,
               $style  = NULL,
               $defer  = FALSE)
{
        /*
         * image
         *
         * binary safe, utf-8 compliant
         */

        $return              = ""; //noscript fallback
        if ($defer) $_return = ""; //javascript deferred loading

        //prepare image (scaling, caching, ...)
        $_url = image_process($url, $width, $height);

        $return                      .= " src=\"" .            x($_url) .   "\"";
        if ($defer) $_return         .= " data-defer-src=\"" . x($_url) .   "\"";
        if (nstre($width))
        {
                $return              .= " width=\"" .          x($width) .  "\"";
                if ($defer) $_return .= " width=\"" .          x($width) .  "\"";
        };
        if (nstre($height))
        {
                $return              .= " height=\"" .         x($height) . "\"";
                if ($defer) $_return .= " height=\"" .         x($height) . "\"";
        };

        //generate alternative resolution sources
        if (preg_match("/^(https?|ftp):\/\/(.+\/)?([^\/]+)\.(png|gif|jpe?g)$/i", $url))
        {
                $resolution    = array(320, 640, 960, 1280); //resolution list
                list($_width)  = getimagesize($url);         //original size
                list($__width) = getimagesize($_url);        //processed size
                $srcset        = $_url . " " . $__width . "w,";

                reset($resolution);
                while (list(, $value) = each($resolution))
                {
                        if ($_width < $value)                                       break;    //original is too small
                        if ($__width < $value)                                      break;    //exceeds target size
                        if (($__width !== FALSE) && (abs($__width - $value) < 100)) continue; //already in range

                        //prepare alternative resolution image (scaling, caching, ...)
                        $__url   = image_process($url, $value);
                        $srcset .= x($__url . " " . $value . "w,");
                };

                $return              .= " srcset=\"" . x(substr($srcset, 0, -1)) . "\"" .
                                        " sizes=\"(max-width:" . $__width . "px) 100vw," . $__width . "px\"";
                if ($defer) $_return .= " data-defer-srcset=\"" . x(substr($srcset, 0, -1)) . "\"" .
                                        " data-defer-sizes=\"(max-width:" . $__width . "px) 100vw," . $__width . "px\"";
        };

        //set alt tag automatically if not specified
        $alt = $title;
        if (stre($alt))
                $alt = preg_match("/([^\/]+)\.[0-9a-zA-Z]+$/", urldecode($url), $_alt) ?
                       strtoalphanum($_alt[1]) : "*";

        $return                      .= " alt=\"" .   x($alt) .   "\"";
        if ($defer) $_return         .= " alt=\"" .   x($alt) .   "\"";
        if (nstre($title))
        {
                $return              .= " title=\"" . x($title) . "\"";
                if ($defer) $_return .= " title=\"" . x($title) . "\"";
        };
        if (nstre($style))
        {
                $return              .= " style=\"" . x($style) . "\"";
                if ($defer) $_return .= " style=\"" . x($style) . "\"";
        };

        return $defer ?
               "<noscript>" .
               "<img$return>" .
               "</noscript>" .
               "<img$_return>" :
               "<img$return>";
};
//-----------------------------------------------------------------------------------------------------
function image_exists($url)
{
        return file_exists(CMS_IMAGES_PATH . "$url.png") ||
               file_exists(CMS_IMAGES_PATH . "$url.gif") ||
               file_exists(CMS_IMAGES_PATH . "$url.jpg");
};
//-----------------------------------------------------------------------------------------------------
function image_process($url,
                       $width  = NULL,
                       $height = NULL)
{
        $format      = array("png", "gif", "jpg"); //supported image formats in order of priority
        $jpg_quality = 75;                         //jpg quality setting

        //verify url
        if (! preg_match("/^(?:https?|ftp):\/\/(?:.+\/)?([^\/]+)\.(png|gif|jpe?g)$/i", $url, $_url))
        {
                //check for local image
                reset($format);
                while (list(, $extension) = each($format))
                {
                        if (file_exists(CMS_IMAGES_PATH . "$url.$extension"))
                                return CMS_IMAGES_URL . "$url.$extension";
                };

                return $url;
        };

        //image dimensions
        $width  = (int)$width;
        $height = (int)$height;

        if ($width  < 0) $width  = 0;
        if ($height < 0) $height = 0;

        //limit dimensions
        if ($width  > 2048) $width  = 2048;
        if ($height > 2048) $height = 2048;

        //image extension
        $extension = strtolower($_url[2]);
        if (streq($extension, "jpeg")) $extension = "jpg";

        //cache file path
        $hash = str_pad(dechex(crc32($url)), 8, "0", STR_PAD_LEFT);
        $file = urldecode($_url[1]) . "-$width" . "x$height" . "-$hash" . ".";
        $hash = substr(md5($file), 0, 4);

        if (! mkpath(CMS_DATA_PATH . "image/cache/$hash/")) return $url;

        $file       = "image/cache/$hash/$file";
        $path_cache = CMS_DATA_PATH . $file;

        //check for local source file
        $count = strlen(CMS_ROOT_URL);
        $local = FALSE;

        if (streq(substr($url, 0, $count), CMS_ROOT_URL))
        {
                $path_source = substr($url, $count);
                $path_source = CMS_ROOT_PATH . urldecode($path_source); //source file path
                $local       = file_exists($path_source);
        };

        //seek cached image
        reset($format);
        while (list(, $_extension) = each($format))
        {
                if (($flag = file_exists($path_cache . $_extension)) === FALSE) continue; //no file

                //determine cache timeout
                $time = filemtime($path_cache . $_extension); //timestamp of cached image

                //local source
                if ($local)
                {
                        $_time = filemtime($path_source); //timestamp of source image
                        $flag  = $time >= $_time;         //TRUE if cache is up to date
                }

                //external source
                else
                {
                        $_time = time(); //current timestamp

                        if ($time < $_time - 120) //cache file is older than 2 minutes
                        {
                                touch($path_cache . $_extension, $_time); //update cache file timestamp

                                if (! isset($__time))
                                {
                                        if (cms_load("html"))
                                        {
                                                //request http header from source location
                                                if (is_array($array = html_header($url)))
                                                {
                                                        //determine source timestamp
                                                        if (isset($array["last-modified"]))
                                                        {
                                                                $__time  = isset($array["date"]) ?
                                                                          $_time - strtotime($array["date"]) : 0;
                                                                $__time += strtotime($array["last-modified"]);
                                                        }

                                                        //no data
                                                        else
                                                        {
                                                                $__time = $_time - 86400; //set to 1 day before now
                                                        };
                                                }
                                                else
                                                {
                                                        $__time = -1; //source missing => cache will never expire
                                                };
                                        }

                                        //missing library
                                        else
                                        {
                                                $__time = $_time - 86400; //set to 1 day before now
                                        };
                                };

                                $flag = $time >= $__time; //TRUE if cache is up to date
                        };
                };

                if ($flag) { $extension = $_extension; break; }; //cached image is up to date

                //delete outdated file so it will not be checked again
                unlink($path_cache . $_extension);
        };

        //cached image is missing or out of date
        if ((! $flag) && copy($local ? $path_source : $url, $path_cache . $extension)) //cache image
        {
                //gd support
                $gd_info = extension_loaded("gd") ? gd_info() : NULL;

                switch ($extension)
                {
                case "png":
//.....................................................................................................

                        //check png support
                        if (! isset($gd_info["PNG Support"])) break;
                        if (! $gd_info["PNG Support"])        break;

                        //rescale image using gd library
                        list($_width, $_height) = \getimagesize($path_cache . $extension);
                        if (($width + $height) == 0)    { $width  = $_width;
                                                          $height = $_height; }
                        else
                        {
                                if ($width == 0)          $width  = round($height * $_width  / $_height);
                                elseif ($height == 0)     $height = round($width  * $_height / $_width);
                                if ($width > $_width)   { $height = round($height * $_width  / $width);
                                                          $width  = $_width;  };
                                if ($height > $_height) { $width  = round($width  * $_height / $height);
                                                          $height = $_height; };
                        };

                        $image  = imagecreatetruecolor($width, $height);
                        $_image = imagecreatefrompng($path_cache . $extension);

                        //check for alpha channel
                        $alpha  = imagecolortransparent($_image) !== -1;
                        if (! $alpha)
                        {
                                $png_type = ord(file_get_contents($path_cache . $extension, FALSE, NULL, 25, 1));
                                $alpha    = in_array($png_type, array(4, 6), TRUE);
                        };

                        if ($alpha)
                        {
                                imagesavealpha($image, TRUE);
                                imagealphablending($image, FALSE);
                                $transparent = imagecolorallocatealpha($image, 0, 0, 0, 127);
                                imagefilledrectangle($image, 0, 0, $width - 1, $height - 1, $transparent);
                        };

                        //scale
                        imagecopyresampled($image, $_image,
                                           0,      0,       0,       0,
                                           $width, $height, $_width, $_height);

                        //no alpha channel required => save as jpg if supported
                        if ((! $alpha) && isset($gd_info["JPEG Support"]) && $gd_info["JPEG Support"])
                        {
                                //delete image
                                unlink($path_cache . $extension);

                                //save as jpg
                                $extension = "jpg";
                                imagejpeg($image, $path_cache . $extension, $jpg_quality);
                        }

                        //alpha channel required or no jpg support
                        else
                        {
                                //overwrite image
                                imagepng($image, $path_cache . $extension);
                        };

                        break;

                case "gif":
//.....................................................................................................

                        //rescaling is not advisable for gif
                        break;

                case "jpg":
//.....................................................................................................

                        if (($width == 0) && ($height == 0)) return $url; //do not reprocess

                        list($_width, $_height) = \getimagesize($path_cache . $extension);
                        if ($width == 0)          $width  = round($height * $_width  / $_height);
                        elseif ($height == 0)     $height = round($width  * $_height / $_width);
                        if ($width > $_width)   { $height = round($height * $_width  / $width);
                                                  $width  = $_width;  };
                        if ($height > $_height) { $width  = round($width  * $_height / $height);
                                                  $height = $_height; };

                        //rescale image using gd library
                        if (isset($gd_info["JPEG Support"]) && $gd_info["JPEG Support"])
                        {
                                $image  = imagecreatetruecolor($width, $height);
                                $_image = imagecreatefromjpeg($path_cache . $extension);

                                //scale
                                imagecopyresampled($image, $_image,
                                                   0,      0,       0,       0,
                                                   $width, $height, $_width, $_height);

                                //overwrite image
                                imagejpeg($image, $path_cache . $extension, $jpg_quality);
                                break;
                        };

                        //rescale image using cgi fallback
                        if (preg_match("/^(lin|win)/i", PHP_OS, $os))
                        {
                                $os = strtolower($os[1]);

                                switch ($os)
                                {
                                case "lin":
                                        if (PHP_INT_SIZE === 8) $os = "l64";
                                        break;

                                case "win":
                                        break;
                                };

                                //scale and overwrite image
                                exec(CMS_PATH . "!bin/jpeg.$os.cgi " .
                                     "\"$path_cache$extension\" $width $height \"$path_cache$extension\"",
                                     $output, $return_var);
                        };

                        break;
                };

                $flag = TRUE;
        };

        //set url to cache location
        if ($flag) $url = CMS_DATA_URL . $file . $extension;

        return $url;
};
//-----------------------------------------------------------------------------------------------------
function getimagesize($filename,
                      &$imageinfo = NULL)
{
        $count = strlen(CMS_ROOT_URL);

        if (streq(substr($filename, 0, $count), CMS_ROOT_URL))
        {
                $_filename = substr($filename, $count);
                $_filename = CMS_ROOT_PATH . urldecode($_filename);
                if (file_exists($_filename)) $filename = $_filename;
        };

        return \getimagesize($filename, $imageinfo);
};
?>
