<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function yesno($boolean)
{
        /*
         * returns boolean value as text
         *
         * binary safe, utf-8 compliant
         */

        return $boolean ? CMS_L_COMMON_008 : CMS_L_COMMON_009;
};
//-----------------------------------------------------------------------------------------------------
function option($boolean,
                $value)
{
        /*
         * return value depending on boolean
         *
         * binary safe, utf-8 compliant
         */

        return $boolean ? $value : NULL;
};
//-----------------------------------------------------------------------------------------------------
function get_mime_type($filename = NULL)
{
        /*
         * determine mime type of filename
         *
         * binary safe, utf-8 compliant
         */

        static $list = FALSE;

        //load extension type list
        if ($list === FALSE)
        {
                if ($hfile = fopen(CMS_PATH . "mimetype", "rb"))
                {
                        while (($line = fgets($hfile, 256)) !== FALSE)
                        {
                                $position         = strpos($line, " ");
                                $extension        = substr($line, 0, $position);
                                $type             = substr($line, $position + 1);
                                $list[$extension] = rtrim($type);
                        };

                        fclose($hfile);
                };
        };

        //return list
        if ($filename === NULL) return $list;

        //get file extension
        $extension = utf8_strtolower(substr(strrchr($filename, "."), 1));

        //return mime type
        return isset($list[$extension]) ?
               $list[$extension] :
               "application/octet-stream";
};
//-----------------------------------------------------------------------------------------------------
function get_mime_list()
{
        /*
         * returns list of all registered file extension
         * associated with mime icon
         *
         * binary safe, utf-8 compliant
         */

         $list   = get_mime_type();
         $return = array();

         reset($list);
         while (list($extension, $type) = each($list))
         {
                $icon = get_mime_icon($type, NULL);
                if ($icon !== NULL) $return[$extension] = $icon;
         };

         return $return;
};
//-----------------------------------------------------------------------------------------------------
function get_mime_icon($type,
                       $default = "mimetype/default")
{
        /*
         * determine mime icon of type
         * returns default if not specified
         *
         * binary safe, utf-8 compliant
         */

        $url = "mimetype/$type";
        if (image_exists($url)) return $url;

        $url = "mimetype/" . substr($type, 0, strpos($type, "/"));
        if (image_exists($url)) return $url;

        return $default;
};
//-----------------------------------------------------------------------------------------------------
function in_array_recursive($needle,
                            $haystack,
                            $strict = FALSE)
{
        /*
         * recursively search array
         *
         * binary safe, utf-8 compliant
         */

        $iterator = new \RecursiveIteratorIterator(new \RecursiveArrayIterator($haystack));

        foreach ($iterator AS $element)
        {
                if ($strict)
                {
                        if ($element === $needle) return TRUE;
                }
                elseif ($element == $needle)
                {
                        return TRUE;
                };
        };

        return FALSE;
};
//-----------------------------------------------------------------------------------------------------
function ksort_recursive(&$array)
{
        /*
         * recursively sort array by key
         *
         * binary safe, utf-8 compliant
         */

        ksort($array);

        reset($array);
        while (list($key, $value) = each($array))
        {
                if (is_array($value))
                        ksort_recursive($array[$key]);
        };
};
//-----------------------------------------------------------------------------------------------------
function bitstring($value,
                   $length = NULL)
{
        /*
         * generate string of 0 and 1
         * representing each bit of value
         * up to length
         *
         * binary safe, utf-8 compliant
         */

        $return = "";

        for ($i = 0, $c = strlen($value); $i < $c; $i++)
        {
                $byte = ord(substr($value, $i, 1));

                for ($_i = 7; $_i >= 0; $_i--)
                        $return .= ($byte & pow(2, $_i)) ? "1" : "0";
        };

        return $length ? substr($return, -$length) : $return;
};
//-----------------------------------------------------------------------------------------------------
function set_time_limit($time)
{
        /*
         * set time limit only if not in safe mode
         *
         * binary safe, utf-8 compliant
         */

        if ((! ini_get("safe_mode")) && ($time > ini_get("max_execution_time")))
                \set_time_limit($time);
};
//-----------------------------------------------------------------------------------------------------
function anonymize_ip($address)
{
    /*
     * remove 4th part of an ipv6 or ipv4 address
     *
     * binary safe, utf-8 compliant
     */

    if ($address = inet_pton($address))
    {
            $length = strlen($address) / 4;
            return inet_ntop(substr($address, 0, $length * 3) . str_repeat(chr(0), $length));
    };

    return "0.0.0.0";
};
?>