<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function strtoalphanum($string)
{
        /*
         * removes non-text characters
         * HTML entities are decoded first
         *
         * binary safe, utf-8 compliant
         */

        $string = htmlentities_decode($string);
        $string = preg_replace("/[^\p{L}\p{N}]+/u", " ", $string);

        return stripspaces($string);
};
//-----------------------------------------------------------------------------------------------------
function stripspaces($string,
                     $preserve_newlines = FALSE,
                     $limit_empty_lines = TRUE)
{
        /*
         * replaces sequences of whitespace or control characters with a single space character
         * removing preceding and trailing whitespace
         *
         * binary safe, utf-8 compliant
         */

        $search  = array("/^[\p{C}\p{Z}]+/u", "/[\p{C}\p{Z}]+$/u", "/[\p{C}\p{Z}]+/u");
        $replace = array("",                  "",                  " ");

        if ($preserve_newlines)
        {
                $array  = explode("\n", $string);
                $array  = preg_replace($search, $replace, $array);
                $string = trim(implode("\n", $array), "\n");

                return $limit_empty_lines ? preg_replace("/\n{3,}/", "\n\n", $string) : $string;
        };

        return preg_replace($search, $replace, $string);
};
//-----------------------------------------------------------------------------------------------------
function whitechartostring($string,
                           $replacement = " ")
{
        /*
         * replaces any whitespace or control character
         *
         * binary safe, utf-8 compliant
         */

        return preg_replace("/[\p{C}\p{Z}]/u", $replacement, $string);
};
//-----------------------------------------------------------------------------------------------------
function nl2br($string)
{
        /*
         * converts newlines ("\r\n", "\n") to HTML linebreak
         *
         * binary safe, utf-8 compliant
         */

        $string = str_replace("\r\n", "\n", $string);
        $string = preg_replace("/(?<=\n)\n/", "&nbsp;<br class=\"multiple\">", $string);
        return str_replace("\n", "&nbsp;<br>", $string);
};
//-----------------------------------------------------------------------------------------------------
function limitstr($string,
                  $length    = 255,
                  $separator = ",")
{
        /*
         * returns string of specified length by removing trailing parts
         * separated by delimeter character(s), used for keywords
         *
         * binary safe, utf-8 compliant
         */

        $return = NULL;
        $part   = explode($separator, $string);

        reset($part);
        while (list(, $value) = each($part))
        {
                if ($value = utf8_trim($value))
                {
                        if ($return) $value = $separator . $value;
                        if (utf8_strlen($return . $value) > $length) break;
                        $return .= $value;
                };
        };

        return utf8_substr($return, 0, $length);
};
//-----------------------------------------------------------------------------------------------------
function zerofill($string,
                  $length)
{
        /*
         * expands string to specified length by applying preceding zero characters
         * used by core resource class
         *
         * binary safe, utf-8 compliant
         */

        return $string . str_repeat("\x00", $length - strlen($string));
};
//-----------------------------------------------------------------------------------------------------
function stre($string)
{
        /*
         * checks if string is empty
         *
         * binary safe, utf-8 compliant
         */

        return (string)$string === "";
};
//-----------------------------------------------------------------------------------------------------
function nstre($string)
{
        /*
         * checks if string is not empty
         *
         * binary safe, utf-8 compliant
         */

        return (string)$string !== "";
};
//-----------------------------------------------------------------------------------------------------
function streq($string1,
               $string2)
{
        /*
         * compares string1 and string2 for equality
         *
         * binary safe, utf-8 compliant
         */

        return strcmp($string1, $string2) === 0;
};
//-----------------------------------------------------------------------------------------------------
function nstreq($string1,
                $string2)
{
        /*
         * compares string1 and string2 for inequality
         *
         * binary safe, utf-8 compliant
         */

        return strcmp($string1, $string2) !== 0;
};
//-----------------------------------------------------------------------------------------------------
function strieq($string1,
                $string2)
{
        /*
         * compares string1 and string2 for case-insensitive equality
         *
         * binary safe, utf-8 compliant
         */

        return preg_match("/" . preg_quote($string1, "/") . "/iu", $string2);
};
//-----------------------------------------------------------------------------------------------------
function substri_count($source,
                       $string)
{
        /*
         * counts occurence of string in source, case independent
         *
         * binary safe, utf-8 compliant
         */

        return preg_match_all("/" . preg_quote($string, "/") . "/iu", $source, $array);
};
//-----------------------------------------------------------------------------------------------------
function verify_email($email)
{
        /*
         * verifies email format of string
         *
         * binary safe, utf-8 compliant
         */

        return preg_match("/^.+" .
                          "@" .
                          "[0-9a-zA-ZäöüÄÖÜ][\-\.0-9a-zA-ZäöüÄÖÜ]{1,60}[0-9a-zA-ZäöüÄÖÜ]" .
                          "\." .
                          "[a-zA-Z]{2,4}$/u",
                          $email);
};
//-----------------------------------------------------------------------------------------------------
function unique_id($count = 8)
{
        /*
         * generates random string of given length
         * used for index keys
         *
         * binary safe, utf-8 compliant
         */

        $string = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
        $return = NULL;

        for ($i = 0; $i < $count; $i++)
                $return .= $string[mt_rand(0, 61)];

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function strabridge($string,
                    $length = 40)
{
        /*
         * abridges string to given length
         *
         * binary safe, utf-8 compliant
         */

        $_length       = utf8_strlen($string);
        if ($_length <= $length) return $string;

        $length       -= 1;
        $length_start  = round($length * 65 / 100, 0);
        $length_end    = $length - $length_start;

        return utf8_substr($string, 0, $length_start) .
               "\xE2\x80\xA6" .
               utf8_substr($string, -$length_end);
}
?>
