<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

function translate_url($address,
                       $param     = NULL,
                       $language  = CMS_LANGUAGE,
                       $canonical = FALSE)
{
        /*
         * expands internally used identifiers to full url:
         * directory://{index}(?{query})(#{fragment})
         * content://{index}(?{query})(#{fragment})
         * image://{index}
         * media://{index}
         *
         * binary safe, utf-8 compliant
         */

        if (preg_match("/^(javascript|mailto):\/\//i", $address)) return $address;

        //complete address
        $address = absolute_path(CMS_ACTIVE_URL, $address);

        //break down address
        $_address = analyze_url($address);
        if ($_address === FALSE) return $address;

        //break down querystring
        if ($_address["query"] !== NULL) parse_str($_address["query"], $_param);
        else                             $_param = array();

        //extend query parameters
        $param = is_array($param) ? array_merge($_param, $param) : $_param;

        switch ($_address["scheme"])
        {
        case "http":
        case "https":
//.....................................................................................................
                $address = url($_address["scheme"],
                               $_address["user"],
                               $_address["pass"],
                               $_address["host"],
                               $_address["port"],
                               $_address["path"],
                               substr(cms_param($param), 1),
                               $_address["fragment"]);
                break;

        case "directory":
//.....................................................................................................
                $fragment  = stre($_address["fragment"]) ? ""              : "#" . $_address["fragment"];
                $_language = nstre($language)            ? $language . "." : "";
                $map       = new map("#system/" . $_language . "directory");

                if ($address = $map->get_value($_address["host"]))
                {
                        $address = CMS_ROOT_URL . cms_path_urlencode($address) . cms_param($param) . $fragment;
                        break;
                };

                $param["content_directory_index"] = $address;
                $param["content_display"]         = "directory";
                $param["cms_language"]            = $language;
                $address                          = CMS_MODULES_URL . "content.php" . cms_param($param) . $fragment;
                break;

        case "content":
//.....................................................................................................
                $param["content_index"] = $_address["host"];
                $param["cms_language"]  = $language;
                $fragment               = stre($_address["fragment"]) ? "" : "#" . $_address["fragment"];
                $address                = CMS_MODULES_URL . "content.php" . cms_param($param) . $fragment;
                break;

        case "image":
//.....................................................................................................
                if (! cms_load("image")) break;

                $image = new image();
                if ($image->data->get($address) === NULL) return image_process("no_image");

                //internal linkage
                $value = $image->data->get($address, "filename");
                if ($value === NULL)
                {
                        //external linkage
                        $value = $image->data->get($address, "url");
                        if ($value !== NULL) return $value;

                        //fallback
                        $value = $_address["host"];
                };

                return CMS_DATA_URL . "image/" . r($value);
                break;

        case "media":
//.....................................................................................................
                if (! cms_load("media")) break;

                $media = new media();
                if ($media->data->get($address) === NULL) break;

                //internal linkage
                $value = $media->data->get($address, "filename");
                if ($value === NULL)
                {
                        //external linkage
                        $value = $media->data->get($address, "url");
                        if ($value !== NULL) return $value;

                        //fallback
                        $value = $_address["host"];
                };

                return CMS_DATA_URL . "media/" . r($value);
                break;
        };

        return $address;
};
//-----------------------------------------------------------------------------------------------------
function analyze_url($address)
{
        /*
         * splits url into data segments
         *
         * binary safe, utf-8 compliant
         */

        $array = parse_url($address);
        if ($array === FALSE) return FALSE;

        if (empty($array["scheme"]))   $array["scheme"]   = NULL; $array["relative"] = stre($array["scheme"]);
        if (empty($array["user"]))     $array["user"]     = NULL;
        if (empty($array["pass"]))     $array["pass"]     = NULL;
        if (empty($array["host"]))     $array["host"]     = NULL;
        if (empty($array["port"]))     $array["port"]     = NULL;
        if (empty($array["path"]))     $array["path"]     = NULL;
        if (empty($array["query"]))    $array["query"]    = NULL;
        if (empty($array["fragment"])) $array["fragment"] = NULL;

        //fix for php before 5.4.7
        if (stre($array["host"]) && streq(substr($array["path"], 0, 2), "//"))
        {
                if (($i = strpos($array["path"], "/", 2)) !== FALSE)
                {
                    $array["host"] = substr($array["path"], 2, $i - 2);
                    $array["path"] = substr($array["path"], $i);
                }
                else
                {
                    $array["host"] = substr($array["path"], 2);
                    $array["path"] = NULL;
                };
        };

        return $array;
};
//-----------------------------------------------------------------------------------------------------
function url($scheme   = "",
             $user     = "",
             $pass     = "",
             $host     = "",
             $port     = "",
             $path     = "",
             $query    = "",
             $fragment = "")
{
        /*
         * constructs an url
         *
         * binary safe, utf-8 compliant
         */

        $string = "";

        if (nstre($scheme . $host))
        {
                $string .= "$scheme://";
                if (nstre($user))                $string .= stre($pass) ? "$user@" : "$user:$pass@";
                $string .= $host;
                if (nstre($port))                $string .= ":$port";
                if (substr($path, 0, 1) !== "/") $path    = "/$path";
        };
        if (nstre($path))     $string .= $path;
        if (nstre($query))    $string .= "?$query";
        if (nstre($fragment)) $string .= "#$fragment";

        return $string;
};
//-----------------------------------------------------------------------------------------------------
function absolute_path($source,
                       $target)
{
        /*
         * expands relative to absolute url using specified base
         *
         * binary safe, utf-8 compliant
         */

        if (($_target = analyze_url($target)) === FALSE) return FALSE;
        if ($_target["relative"] === FALSE)              return $target;
        if (($_source = analyze_url($source)) === FALSE) return FALSE;
        if ($_source["relative"] !== FALSE)              return FALSE;

        if (substr($_target["path"], 0, 1) === "/")
                return url($_source["scheme"],
                           $_source["user"],
                           $_source["pass"],
                           $_source["host"],
                           $_source["port"],
                           $_target["path"],
                           $_target["query"],
                           $_target["fragment"]);

        if (stre($_target["path"]))
        {
                $_target["path"] = $_source["path"];
        }
        else
        {
                if (substr($_source["path"], 0, 1) === "/")  $_source["path"] = substr($_source["path"], 1);
                if (substr($_source["path"], -1, 1) !== "/") $_source["path"] = dirname($_source["path"]);
                else                                         $_source["path"] = substr($_source["path"], 0, -1);
                if ($_source["path"] === ".")                $_source["path"] = "";

                $source_path = explode("/", $_source["path"]);
                $target_path = explode("/", $_target["path"]);

                while (list($key, $value) = each($target_path))
                {
                        if ($value === "..")    array_pop($source_path);
                        elseif ($value !== ".") array_push($source_path, $value);
                };

                $_target["path"] = implode("/", $source_path);
        };

        return url($_source["scheme"],
                   $_source["user"],
                   $_source["pass"],
                   $_source["host"],
                   $_source["port"],
                   $_target["path"],
                   $_target["query"],
                   $_target["fragment"]);
};
//-----------------------------------------------------------------------------------------------------
function querystring($param,
                     $alter = NULL)
{
        /*
         * generates querystring
         *
         * binary safe, utf-8 compliant
         */

        if (! is_array($param)) return FALSE;
        if (is_array($alter))   $param = array_merge($param, $alter);

        $return = "";

        //recursively retrieve field names and values
        $array = array();
        $func  = function($value, $key = "") use (&$func, &$array)
        {
                if (is_array($value))
                {
                    ksort($value);
                    reset($value);
                    while (list($_key, $_value) = each($value))
                        $func($_value, stre($key) ? $_key : $key . "[$_key]");
                }
                else
                {
                    $array[$key] = $value;
                };
        };

        $func($param);

        reset($array);
        while (list($key, $value) = each($array))
        {
                if (stre($value)) continue;

                $value   = (substr($value, 0, 1) === "\x1B") ? substr($value, 1) : r($value);
                $return .= ($return ? "&" : "?") . r($key) . "=$value";
        };

        return $return;
};
//-----------------------------------------------------------------------------------------------------
function import_querystring($address)
{
        /*
         * import values in querystring as variables
         *
         * binary safe, utf-8 compliant
         */

        $address = parse_url($address);
        if (! isset($address["query"])) return FALSE;

        parse_str($address["query"], $param);

        reset($param);
        while (list($key, $value) = each($param))
        {
                $value         = cms_utf8_normalize($value);
                $_GET[$key]    = $value;
                $GLOBALS[$key] = $value;
        };

        return TRUE;
};
?>