<?php /*

   nuos Content Management System
   Copyright (C) 2012 Patrick Heyer, www.nuos-cms.org

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

namespace cms;

//-----------------------------------------------------------------------------------------------------
//===< HTML CLASS >====================================================================================
//-----------------------------------------------------------------------------------------------------

//constants
define("CMS_HTML_TIMEOUT_TOTAL",      10);
define("CMS_HTML_TIMEOUT_CHUNK",       3);
define("CMS_HTML_SIZE_CHUNK",     524289);
define("CMS_HTML_LIMIT",         1048576);
define("CMS_HTML_GET_FIRST",           1);
define("CMS_HTML_GET_NEXT",            2);

//related functions
//.....................................................................................................
function html_fopen($url,
                    $get_header_only = FALSE)
{
        //break address into components
        $url = analyze_url($url);
        if ($url["host"] === NULL) return FALSE; //no host

        //scheme
        switch (strtolower($url["scheme"]))
        {
        case "":
        case "http":
                $transport    = "tcp://";
                $port_default = 80;
                break;

        case "https":
                $transport    = "ssl://";
                $port_default = 443;
                break;

        default:
                return FALSE; //impermissible scheme
        };

        //address components
        $host  = $url["host"];
        $path  = ($url["path"] === NULL)  ? "/"           : $url["path"];
        $port  = ($url["port"] === NULL)  ? $port_default : (int)$url["port"];
        $query = ($url["query"] === NULL) ? ""            : "?" . $url["query"];

        //authorization
        $authorization = "";

        if ($url["user"] !== NULL)
        {
                $authorization = $url["user"];
                if (isset($url["pass"])) $authorization .= ":" + $url["pass"];
                $authorization = "Authorization: Basic " + base64_encode($authorization) . "\r\n";
        };

        //connect
        $hfile = @stream_socket_client($transport . $host . ":$port", $error_number, $error, CMS_HTML_TIMEOUT_TOTAL);
        if ($hfile === FALSE) return FALSE; //connection error

        //send request
        $command = $get_header_only ? "HEAD" : "GET";
        $status  = html_send(
                $hfile,
                "$command $path$query HTTP/1.0\r\n" .
                "Host: $host\r\n" .
                "User-Agent: " . CMS_SOFTWARE . ", " . CMS_HOST . "\r\n" .
                $authorization .
                "Connection: close\r\n" .
                "Accept: text/html\r\n" .
                "\r\n");

        if ($status === FALSE) { fclose($hfile); return FALSE; }; //error

        //receive http status code
        $header = html_fetch_header($hfile);
        if ($header === FALSE) { fclose($hfile); return FALSE; }; //error

        //check http status code
        if (! preg_match("/^HTTP\/[0-9]\.[0-9]\s+([1-4][0-9]{2})/i", $header, $_header))
        {
                //invalid http status
                fclose($hfile);
                return $get_header_only ? $header : FALSE;
        };

        //store status
        $return = array("#status" => $_header[1]);

        //retrieve header fields
        while (($header = html_fetch_header($hfile)) !== FALSE)
        {
                //check for end of header (empty line)
                if ($header === "") break; //end of header

                //keep header field
                if ($offset = strpos($header, ":"))
                        $return[strtolower(substr($header, 0, $offset))] =
                                ltrim(substr($header, $offset + 1));
        };

        //return header information
        if ($get_header_only) { fclose($hfile); return $return; };

        //return connection
        return $hfile;
};
//-----------------------------------------------------------------------------------------------------
function html_send($hfile,
                   $data)
{
        $length = strlen($data); //data size
        $time   = time();        //start time

        stream_set_blocking($hfile, FALSE);
        stream_set_timeout($hfile, CMS_HTML_TIMEOUT_CHUNK);

        while ($length > 0)
        {
                //send data
                $_length = fwrite($hfile, $data);
                if ($_length === FALSE) return FALSE; //error

                //check time
                if ((time() - $time) > CMS_HTML_TIMEOUT_CHUNK) return FALSE; //timeout

                //wait
                usleep(100000);

                //extract remaining data
                $length -= $_length;
                $data    = substr($data, -$length);
        };

        return TRUE;
};
//-----------------------------------------------------------------------------------------------------
function html_fetch_header($hfile)
{
        $time = time(); //start time

        stream_set_blocking($hfile, FALSE);
        stream_set_timeout($hfile, CMS_HTML_TIMEOUT_CHUNK);
        $meta = stream_get_meta_data($hfile);

        while ((! feof($hfile)) && (! $meta["timed_out"]))
        {
                //retrieve single line
                $buffer = stream_get_line($hfile, CMS_HTML_SIZE_CHUNK, "\n");
                if ($buffer === FALSE) continue;

                //data retrieved
                if ($buffer !== "") return trim($buffer); //return data

                //check time
                if ((time() - $time) > CMS_HTML_TIMEOUT_CHUNK) return FALSE; //timeout

                //wait
                usleep(100000);

                $meta = stream_get_meta_data($hfile);
        };

        return FALSE;
};
//-----------------------------------------------------------------------------------------------------
function html_fetch_data($hfile)
{
        $buffer = "";     //data buffer
        $time   = time(); //start time

        stream_set_blocking($hfile, FALSE);
        stream_set_timeout($hfile, CMS_HTML_TIMEOUT_CHUNK);
        $meta = stream_get_meta_data($hfile);

        while ((! feof($hfile)) && (! $meta["timed_out"]))
        {
                //retrieve data chunk
                $_buffer = stream_get_contents($hfile, CMS_HTML_SIZE_CHUNK);
                if ($_buffer === FALSE) return FALSE; //error

                //append to buffer
                $buffer .= $_buffer;

                //check time
                if ((time() - $time) > CMS_HTML_TIMEOUT_CHUNK) return FALSE; //timeout

                //wait
                usleep(100000);

                $meta = stream_get_meta_data($hfile);
        };

        return $meta["timed_out"] ? FALSE : $buffer;
};
//-----------------------------------------------------------------------------------------------------
function html_header($url)
{
        return html_fopen($url, TRUE);
};
//-----------------------------------------------------------------------------------------------------
function html_page_data($url)
{
        //open connection
        $hfile = html_fopen($url);
        if ($hfile === FALSE) return FALSE; //error

        //retrieve data
        $data  = html_fetch_data($hfile);
        fclose($hfile);

        return $data;
};
//-----------------------------------------------------------------------------------------------------
function html_page_info($url)
{
        $html = new html($url);

        if (stre($html->file)) return FALSE;

        //check for basic html document structure
        if (! preg_match("/^.*?<html(?:\s[^>]*)?>" .
                         ".*?<head(?:\s[^>]*)?>.*?<\/head\s*>" .
                         ".*?<body(?:\s[^>]*)?>.*?<\/body\s*>" .
                         ".*?<\/html\s*>.*?$/isu",
                         $html->file))
                return FALSE;

        //get meta data
        $html->reset();
        while ($buffer = $html->get(CMS_HTML_GET_NEXT, "meta", FALSE))
        {
                if (
                     (
                       isset($buffer["#attribute"]["name"])
                       ||
                       isset($buffer["#attribute"]["http-equiv"])
                     )
                     &&
                     isset($buffer["#attribute"]["content"])
                   )
                {
                        $key   = isset($buffer["#attribute"]["name"]) ?
                                 $buffer["#attribute"]["name"] :
                                 $buffer["#attribute"]["http-equiv"];
                        $key   = stripspaces(utf8_strtolower(htmlentities_decode($key)));
                        $value = stripspaces(htmlentities_decode($buffer["#attribute"]["content"]));

                        if (isset($return[$key])) $return[$key] .= ", $value";
                        else                      $return[$key]  = $value;
                };
        };

        //get title
        $buffer          = $html->get(CMS_HTML_GET_FIRST, "title");
        $return["title"] = stripspaces(htmlentities_decode($buffer["#pcdata"]));

        //get content
        $buffer         = $html->get(CMS_HTML_GET_FIRST, "body");
        $return["body"] = htmltoplain($buffer["#pcdata"]);

        //get document base
        if (
             ($buffer = $html->get(CMS_HTML_GET_FIRST, "base", NULL))
             &&
             isset($buffer["#attribute"]["href"])
           )
        {
                $_url = htmlentities_decode($buffer["#attribute"]["href"]);
                $url  = absolute_path($url, $_url);
        };

        //get links
        $return["links"] = array();

        $html->reset();
        while ($buffer = $html->get(CMS_HTML_GET_NEXT, "a"))
        {
                if (! isset($buffer["#attribute"]["href"])) continue; //no target address

                if (
                     isset($buffer["#attribute"]["rel"])
                     &&
                     preg_match("/\bnofollow\b/i", $buffer["#attribute"]["rel"])
                   )
                        continue; //ignore link

                $_url = htmlentities_decode($buffer["#attribute"]["href"]);

                //verify address
                if (($_buffer = analyze_url($_url)) === FALSE) continue; //invalid url
                $_buffer["scheme"] = strtolower($_buffer["scheme"]);

                switch ($_buffer["scheme"])
                {
                case "":
                case "http":
                case "https":

                        //add to list
                        $_url              = absolute_path($url, $_url);
                        $return["links"][] = array("url"  => $_url,
                                                   "text" => $buffer["#pcdata"]);
                        break;
                };
        };

        return $return;
};

class html
{
//properties
//-----------------------------------------------------------------------------------------------------
        public $file     = NULL;
        public $position = 0;
//constructor
//-----------------------------------------------------------------------------------------------------
        function __construct($url)
        {
                //retrieve data
                $hfile      = html_fopen($url);
                if ($hfile === FALSE) return; //no connection
                $this->file = html_fetch_data($hfile);
                fclose($hfile);

                if ($this->file === FALSE)
                {
                        $this->file = NULL;
                        return;
                };

                //convert character set
                if (! utf8_detect($this->file)) //not utf-8
                {
                        $flag = TRUE;

                        //search for content-type
                        if (
                             preg_match("/<meta\s[^<>]*http-equiv\s*=\s*((?:\"|')?)\s*content-type\s*(?:(?:\\1|\s)[^<>]*>|\\1\/?>)/is",
                                        $this->file, $array)
                             &&
                             preg_match("/<meta\s[^<>]*content\s*=\s*((?:\"|')?)\s*([^<>]+)\s*(?:(?:\\1|\s)[^<>]*>|\\1\/?>)/is",
                                        $array[0], $array)
                             &&
                             cms_load("mime")
                           )
                        {
                                $content_type = mime_extract_rfc2822_header($array[2]);

                                if (isset($content_type["charset"]))
                                {
                                        $this->file = utf8_convert($this->file, $content_type["charset"]);
                                        $flag       = FALSE;
                                };
                        };

                        //not found, use default conversion
                        if ($flag) $this->file = utf8_convert($this->file);
                };

                //normalize
                $this->file = utf8_normalize($this->file);

                //remove noise
                $search     = array("/<!--.*?-->/su",                            //comment
                                    "/<script(?:\s[^>]*)?>.*?<\/script\s*>/isu", //script
                                    "/<style(?:\s[^>]*)?>.*?<\/style\s*>/isu");  //style
                $this->file = preg_replace($search, "", $this->file);
        }
//methods
//-----------------------------------------------------------------------------------------------------
        function get_attributes($string)
        {
                $return   = FALSE;
                $position = 0;

                while (TRUE)
                {
                        //attribute = "value"
                        if (preg_match("/[\p{C}\p{Z}]*([^\p{C}\p{Z}=]+)[\p{C}\p{Z}]*=[\p{C}\p{Z}]*\"([^\"]*)\"/Uu",
                                       $string,
                                       $array,
                                       PREG_OFFSET_CAPTURE,
                                       $position))
                        {
                                $return[$array[1][0]] = $array[2][0];
                                $position             = $array[0][1] + strlen($array[0][0]);
                                continue;
                        };

                        //attribute = 'value'
                        if (preg_match("/[\p{C}\p{Z}]*([^\p{C}\p{Z}=]+)[\p{C}\p{Z}]*=[\p{C}\p{Z}]*'([^']*)'/Uu",
                                       $string,
                                       $array,
                                       PREG_OFFSET_CAPTURE,
                                       $position))
                        {
                                $return[$array[1][0]] = $array[2][0];
                                $position             = $array[0][1] + strlen($array[0][0]);
                                continue;
                        };

                        //attribute = value
                        if (preg_match("/[\p{C}\p{Z}]*([^\p{C}\p{Z}=]+)[\p{C}\p{Z}]*=[\p{C}\p{Z}]*([^\p{C}\p{Z}]*)/Uu",
                                       $string,
                                       $array,
                                       PREG_OFFSET_CAPTURE,
                                       $position))
                        {
                                $return[$array[1][0]] = $array[2][0];
                                $position             = $array[0][1] + strlen($array[0][0]);
                                continue;
                        };

                        //attribute
                        if (preg_match("/[\p{C}\p{Z}]*([^\p{C}\p{Z}]+)/Uu",
                                       $string,
                                       $array,
                                       PREG_OFFSET_CAPTURE,
                                       $position))
                        {
                                $return[$array[1][0]] = TRUE;
                                $position             = $array[0][1] + strlen($array[0][0]);
                                continue;
                        };

                        break;
                };

                return $return;
        }
//-----------------------------------------------------------------------------------------------------
        function get($option     = CMS_HTML_GET_NEXT,
                     $element    = NULL,
                     $get_pcdata = TRUE)
        {
                if (! $this->file)                 return FALSE;
                if ($option == CMS_HTML_GET_FIRST) $this->position = 0;
                if ($element === NULL)             $element        = "[a-z0-9]+";
                else                               $element        = addcslashes(preg_quote($element), "/");

                //search for element start
                if (preg_match("/<(" . $element . ")(?:[\p{C}\p{Z}]+(.*))?(\/?)>/isUu",
                               $this->file,
                               $array,
                               PREG_OFFSET_CAPTURE,
                               $this->position))
                {
                        $return["#element"]                  = utf8_strtolower($array[1][0]);

                        //get attributes
                        if ($array[2]) $return["#attribute"] = $this->get_attributes($array[2][0]);

                        $return["#pcdata"]                   = "";
                        $this->position                      = $array[0][1] + strlen($array[0][0]);

                        //element
                        if ($array[3][0] || (! $get_pcdata)) return $return;

                        //search for element end
                        if (preg_match("/<\/[\p{C}\p{Z}]*" . $element . "[\p{C}\p{Z}]*>/iUu",
                                       $this->file,
                                       $array,
                                       PREG_OFFSET_CAPTURE,
                                       $this->position))
                        {
                                //get pcdata
                                $return["#pcdata"] = substr($this->file, $this->position, $array[0][1] - $this->position);

                                $this->position    = $array[0][1] + strlen($array[0][0]);
                        };

                        return $return;
                };

                return NULL;
        }
//-----------------------------------------------------------------------------------------------------
        function reset()
        {
                if (! $this->file) return FALSE;

                $this->position = 0;

                return TRUE;
        }
};
?>
